<?php

// CUSTOM CONTROLS

if ( class_exists( 'WP_Customize_Control' ) ) {

	// MULTI CHECKBOX

	class FUPI_Customize_Multi_Checkbox extends WP_Customize_Control {

		public $type = 'multi_checkbox';

		public function render_content() {

			if ( empty( $this->choices ) )
			return; ?>

			<?php if ( !empty( $this->label ) ) : ?>
				<span class="customize-control-title"><?php echo esc_html( $this->label ); ?></span>
			<?php endif; ?>

			<?php if ( !empty( $this->description ) ) : ?>
				<span class="description customize-control-description"><?php echo $this->description; ?></span>
			<?php endif; ?>

			<?php $multi_values = !is_array( $this->value() ) ? explode( ',', $this->value() ) : $this->value(); ?>

			<ul>
				<?php foreach ( $this->choices as $value => $label ) : ?>

					<li>
						<label>
							<input type="checkbox" value="<?php echo esc_attr( $value ); ?>" <?php checked( in_array( $value, $multi_values ) ); ?> />
							<?php echo esc_attr( $label ); ?>
						</label>
					</li>

				<?php endforeach; ?>
			</ul>

			<input type="hidden" class="fupi_control_multi_checkbox" <?php $this->link(); ?> value="<?php echo esc_attr( implode( ',', $multi_values ) ); ?>" />
		<?php }
	}
};

// SECTION

$wp_customize->add_section( 'fupi_set_cookie_notice',
   array(
      'title' => esc_html__( 'Cookie Notice', 'fupi' ),
      'active_callback' => '', // Rarely needed
   )
);

// SETTINGS START

$wp_customize->add_setting( 'fupi_cookie_notice[position]', array( 'type' => 'option', 'sanitize_callback' => 'sanitize_key', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[position]',
    array(
        'label' => esc_html__( 'Position', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'select',
        'choices' => array(
            'bottom' => esc_html__( 'Bottom - Narrow', 'fupi' ),
			'bottom_wide' => esc_html__( 'Bottom - Wide', 'fupi' ),
			'bottom_full' => esc_html__( 'Bottom - Very Wide', 'fupi' ),
            'leftcorner' => esc_html__( 'Bottom Left', 'fupi' ),
            'popup' => esc_html__( 'Center', 'fupi' ),
        )
    )
);

$wp_customize->add_setting( 'fupi_cookie_notice[paddings]', array( 'type' => 'option', 'default' => array('default'), 'sanitize_callback' => 'sanitize_key', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[paddings]',
    array(
        'label' => esc_html__( 'Paddings', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'select',
        'choices' => array(
            'default' => esc_html__( 'Default', 'fupi' ),
			'compact' => esc_html__( 'Compact', 'fupi' ),
			'supercompact' => esc_html__( 'Super compact', 'fupi' ),
        ),
		'default' => 'default',
    )
);

// BUTTON PLACEMENT

$wp_customize->add_setting( 'fupi_cookie_notice[btn_config]', array( 'type' => 'option', 'sanitize_callback' => 'sanitize_key', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[btn_config]',
    array(
        'label' => esc_html__( 'Button placement & configuration', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'select',
        'choices' => array(
            'default' => esc_html__( 'Default', 'fupi' ),
            'config_1' => esc_html__( 'Inline', 'fupi' ),
            'config_2' => esc_html__( 'Inline reversed', 'fupi' ),
        )
    )
);


// BUTTON PLACEMENT

$wp_customize->add_setting( 'fupi_cookie_notice_size', array( 'sanitize_callback' => 'sanitize_key', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice_size',
    array(
        'label' => esc_html__( 'Button size', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'select',
        'choices' => array(
            'default' => esc_html__( 'Default (set by your theme)', 'fupi' ),
            'small' => esc_html__( 'Small', 'fupi' ),
            'medium' => esc_html__( 'Medium', 'fupi' ),
			'large' => esc_html__( 'Large', 'fupi' ),
        ),
		'description' => esc_html__( 'The size of text is controlled by your theme.', 'fupi' ),
    )
);

// HIDE BTNS & SECTIONS

$wp_customize->add_setting( 'fupi_cookie_notice[hide]', array( 'type' => 'option', 'default' => array(), 'sanitize_callback' => array($this, 'fupi_customizer_sanitize'), 'transport' => 'postMessage' ) );
$wp_customize->add_control( new FUPI_Customize_Multi_Checkbox( $wp_customize, 'fupi_cookie_notice[hide]',
    array(
		'label' => esc_html__( 'Show/Hide elements', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'choices' => array(
			'decline_btn' => esc_html__( 'Hide "decline" button (does not comply with cookie laws)', 'fupi' ),
            'settings_btn' => esc_html__( 'Hide "cookie settings" button', 'fupi' ),
            'stats' => esc_html__( 'Hide "statistics" section in "cookie settings"', 'fupi' ),
            'market' => esc_html__( 'Hide "marketing" section in "cookie settings"', 'fupi' ),
			'pers'=> esc_html__( 'Hide "personalisation" section in "cookie settings"', 'fupi' ),
			'powered_by'=> esc_html__( 'Show "Powered by" link', 'fupi' ),
		),
	)
) );

// OVERLAY

$wp_customize->add_setting( 'fupi_cookie_notice[overlay]', array( 'type' => 'option', 'sanitize_callback' => array($this, 'fupi_customizer_sanitize'), 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[overlay]',
    array(
        'label' => esc_html__( 'Add a background overlay behing a centrally-positioned notice', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'checkbox',
    )
);

// SCROLL LOCK

$scroll_lock_opts = array(
	'label' => esc_html__( 'Lock page scroll until visitors make choice', 'fupi' ),
	'section' => 'fupi_set_cookie_notice',
	'type' => 'checkbox',
);

$wp_customize->add_setting( 'fupi_cookie_notice[scroll_lock]', array( 'type' => 'option', 'sanitize_callback' => array($this, 'fupi_customizer_sanitize'), 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[scroll_lock]', $scroll_lock_opts );

// PAGE BLUR

$wp_customize->add_setting( 'fupi_cookie_notice[blur_page]', array( 'type' => 'option', 'sanitize_callback' => array($this, 'fupi_customizer_sanitize'), 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[blur_page]',
    array(
        'label' => esc_html__( 'Blur page until visitors make choice', 'fupi' ),
		'description' => esc_html__( 'To prevent some elements from bluring add to them the CSS class "fupi_noblur". It works only on elements that are direct descendants of <body> tag.', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'checkbox',
    )
);

// STYLING

$wp_customize->add_setting( 'fupi_cookie_notice[btn_class]', array( 'type' => 'option', 'sanitize_callback' => array($this, 'fupi_customizer_sanitize'), 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[btn_class]',
    array(
        'label' => esc_html__( 'Button class', 'fupi' ),
        'description' => esc_html__( '(Advanced) This class will be applied to all buttons in the cookie notice panel', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'text',
    )
);

$wp_customize->add_setting( 'fupi_cookie_notice[cta_class]', array( 'type' => 'option', 'sanitize_callback' => array($this, 'fupi_customizer_sanitize'), 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[cta_class]',
    array(
        'label' => esc_html__( 'Call-to-action button class', 'fupi' ),
        'description' => esc_html__( 'This class will be applied only to the "Agree" and "Agree to selected" buttons in the cookie notice panel', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'text',
    )
);

// ROUNDED CORNERS IN NOTICE

$wp_customize->add_setting( 'fupi_notice_round_corners', array( 'sanitize_callback' => 'sanitize_key', 'transport' => 'postMessage', 'default' => 0 ) );
$wp_customize->add_control( 'fupi_notice_round_corners',
    array(
        'label' => esc_html__( 'Rounded corners of notice pannels', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'number',
		'default' => 0,
    )
);

// ROUNDED CORNERS IN BUTTONS

$wp_customize->add_setting( 'fupi_notice_btn_round_corners', array( 'sanitize_callback' => 'sanitize_key', 'transport' => 'postMessage', 'default' => 0 ) );
$wp_customize->add_control( 'fupi_notice_btn_round_corners',
    array(
        'label' => esc_html__( 'Rounded corners of buttons ', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'number',
		'default' => 0,
    )
);

// COLORS

$wp_customize->add_setting( 'fupi_notice_bg_color', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_bg_color',
	array(
		'label' => esc_html__( 'Background Color', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_h_color', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_h_color',
	array(
		'label' => esc_html__( 'Headline Color', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_text_color', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_text_color',
	array(
		'label' => esc_html__( 'Text Color', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_cta_color', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_cta_color',
	array(
		'label' => esc_html__( 'Call-to-action Button Color', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_cta_color_hover', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_cta_color_hover',
	array(
		'label' => esc_html__( 'Call-to-action Button Color (Hover)', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_cta_txt_color', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_cta_txt_color',
	array(
		'label' => esc_html__( 'Call-to-action Button Text Color', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_cta_txt_color_hover', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_cta_txt_color_hover',
	array(
		'label' => esc_html__( 'Call-to-action Button Text Color (Hover)', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_btn_color', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_btn_color',
	array(
		'label' => esc_html__( 'Button Color', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_btn_color_hover', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_btn_color_hover',
	array(
		'label' => esc_html__( 'Button Color (Hover)', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_btn_txt_color', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_btn_txt_color',
	array(
		'label' => esc_html__( 'Button Text Color', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_btn_txt_color_hover', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_btn_txt_color_hover',
	array(
		'label' => esc_html__( 'Button Text Color (Hover)', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_notice_switch_color', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_switch_color',
	array(
		'label' => esc_html__( 'Color of enabled switches', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

$wp_customize->add_setting( 'fupi_cookie_notice_border', array( 'sanitize_callback' => 'sanitize_key', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice_border',
   array(
      'label' => esc_html__( 'Border style' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'radio',
      'choices' => array(
         'none' => esc_html__( 'None', 'fupi' ),
         'small_shadow' => esc_html__( 'Small shadow', 'fupi' ),
         'large_shadow' => esc_html__( 'Large shadow', 'fupi' ),
         'thin_border' => esc_html__( 'Thin Border', 'fupi' ),
         'wide_border' => esc_html__( 'Wide Border', 'fupi' ),
	 ),
   )
);

$wp_customize->add_setting( 'fupi_notice_border_color', array( 'sanitize_callback' => 'sanitize_hex_color', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_notice_border_color',
	array(
		'label' => esc_html__( 'Border color', 'fupi' ),
		'section' => 'fupi_set_cookie_notice',
		'type' => 'color',
	)
);

// PRIVACY POLICY PAGE ID

$wp_customize->add_setting( 'fupi_cookie_notice[privacy_page_id]', array( 'type' => 'option', 'sanitize_callback' => 'absint', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[privacy_page_id]',
    array(
        'label' => esc_html__( 'Privacy policy page', 'fupi' ),
        'section' => 'fupi_set_cookie_notice',
        'type' => 'dropdown-pages',
    )
);

// TEXTS

$wp_customize->add_setting( 'fupi_cookie_notice[notif_text]', array( 'type' => 'option', 'sanitize_callback' => 'wp_filter_nohtml_kses', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[notif_text]',
   array(
      'label' => esc_html__( 'Notification text', 'fupi' ),
      'description' => esc_html__( 'Words wrapper with {{ }} will turn into a link to your privacy policy page', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'textarea',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[agree_text]', array( 'type' => 'option', 'sanitize_callback' => 'sanitize_text_field', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[agree_text]',
   array(
      'label' => esc_html__( 'Text in "Agree" button', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'text',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[agree_to_selected_text]', array( 'type' => 'option', 'sanitize_callback' => 'sanitize_text_field', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[agree_to_selected_text]',
   array(
      'label' => esc_html__( 'Text in "Agree to selected" button', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'text',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[decline_text]', array( 'type' => 'option', 'sanitize_callback' => 'sanitize_text_field', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[decline_text]',
   array(
      'label' => esc_html__( 'Text in "Decline" button', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'text',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[cookie_settings_text]', array( 'type' => 'option', 'sanitize_callback' => 'wp_filter_nohtml_kses', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[cookie_settings_text]',
   array(
      'label' => esc_html__( 'Text in "Cookie settings" button', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'text',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[return_text]', array( 'type' => 'option', 'sanitize_callback' => 'sanitize_text_field', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[return_text]',
   array(
      'label' => esc_html__( 'Text in "Return" button', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'text',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[stats_headline_text]', array( 'type' => 'option', 'sanitize_callback' => 'sanitize_text_field', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[stats_headline_text]',
   array(
      'label' => esc_html__( '"Statistics" section title', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'text',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[stats_text]', array( 'type' => 'option', 'sanitize_callback' => 'wp_filter_nohtml_kses', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[stats_text]',
   array(
      'label' => esc_html__( '"Statistics" section description', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'textarea',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[pers_headline_text]', array( 'type' => 'option', 'sanitize_callback' => 'sanitize_text_field', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[pers_headline_text]',
   array(
      'label' => esc_html__( '"Personalisation" section title', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'text',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[pers_text]', array( 'type' => 'option', 'sanitize_callback' => 'wp_filter_nohtml_kses', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[pers_text]',
   array(
      'label' => esc_html__( '"Personalisation" section description', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'textarea',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[marketing_headline_text]', array( 'type' => 'option', 'sanitize_callback' => 'sanitize_text_field', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[marketing_headline_text]',
   array(
      'label' => esc_html__( '"Marketing" section title', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'text',
   )
);

$wp_customize->add_setting( 'fupi_cookie_notice[marketing_text]', array( 'type' => 'option', 'sanitize_callback' => 'wp_filter_nohtml_kses', 'transport' => 'postMessage' ) );
$wp_customize->add_control( 'fupi_cookie_notice[marketing_text]',
   array(
      'label' => esc_html__( '"Marketing" section description', 'fupi' ),
	  'description' => esc_html__( 'Leave empty to use default', 'fupi' ),
      'section' => 'fupi_set_cookie_notice',
      'type' => 'textarea',
   )
);
