<?php

// fupi_field_html($recipe, $values, $arr_id)
// -	$recipe 		array 		no default			recipe for setting up a single field
// -	$field_id 		string  	default: false 		field's name and ID (optional - passed by r3)
// -	$saved_value 	array 		default: false 		saved value of this field (optional - passed by r3)
//
// ! important - remember that this code generates a SINGLE field (or a repeater / multi field) and not ALL fields

// GET SAVED VALUES
// if we don't have any - values are passed by r3 repeater

if ( empty( $field_id ) ) {

	// if id of options array is given (for serialized data)
	if( isset( $recipe['option_arr_id'] ) ) {

		// get the options array (returns false if not exists)
		$saved_data_arr = get_option( $recipe['option_arr_id'] );

		if( ! empty ($saved_data_arr) && isset( $saved_data_arr[$recipe['field_id']] )) { // if options array and field value exist
			$saved_value = $saved_data_arr[$recipe['field_id']]; // get the value of the field we are about to generate
		} else { // if array doesn't exist
			$saved_value = isset( $recipe['default'] ) ? $recipe['default'] : false; // set default or nothing
		};

		$field_id = $recipe['option_arr_id'] . '['.$recipe['field_id'].']'; // get correct field id

	// for unserialized data (not used anywhere in the plugin)
	} else {
		$saved_value = get_option( $recipe['field_id'] ); // get the current field value (returns false if not exists)
		if( ! $saved_value ) $value = isset( $recipe['default'] ) ? $recipe['default'] : false; // set default or nothing
		$field_id = $recipe['field_id'];
	}
} else {
	$is_r3 = true;
}

// GET DATA COMMON TO MOST FIELDS

$placeholder 		= isset( $recipe['placeholder'] ) ? $recipe['placeholder'] : '';
$el_class 			= ! empty ( $recipe['el_class'] ) ? $recipe['el_class'] : '';
if ( isset( $recipe['required'] ) ) $el_class .= ' fupi_req';

$el_data_target 	= ! empty ( $recipe['el_data_target'] ) ? $recipe['el_data_target'] : '';
$data_format		= ! empty( $recipe['format'] ) ? 'data-dataformat="' . $recipe['format'] . '"': '';
// if ( !empty ($data_format) ) trigger_error('data format ' . $data_format);

// BUILD HTML

// BEFORE FIELD

if( isset( $recipe['before field'] ) ){
	printf( '<span class="before-field">%s</span>', $recipe['before field'] ); // Show it
}

// Check which type of field we want
switch ( $recipe['type'] ){

	// =======

	case 'r3':

		$fields	= isset( $recipe['fields'] ) && is_array( $recipe['fields'] ) ? $recipe['fields'] : false;
		if ( ! $fields ) return;

		$repeater 		= ! empty( $recipe['is_repeater'] );
		$el_class 	   .= $repeater ? ' fupi_r3_repeater' : '';
		$btns_class 	= ! empty ( $recipe['btns_class'] ) ? $recipe['btns_class'] : '';
		$sections_nr 	= $repeater && is_array( $saved_value ) ? count( $saved_value ) : 0;
		$section_class	= ! empty( $recipe['wrap_class'] ) ? $recipe['wrap_class'] : '';
		$i = 0;

		for ( $i=0; $i <= $sections_nr; $i++) {

			// here we want to show empty section if there is no data yet or we want to remove an empty section if it doesn't have any values
			if ( empty( $saved_value[$i] ) && $sections_nr > 0 ) continue;

			echo '<div class="fupi_r3_section '.$el_class.' '. $section_class .'">';

				foreach( $fields as $field_recipe ) {

					$f_id 			= ! empty( $field_recipe['field_id'] ) ? $field_recipe['field_id'] : '';
					$f_default  	= ! empty( $field_recipe['default'] ) ? $field_recipe['default'] : '';
					$f_label 		= ! empty( $field_recipe['label'] ) ? $field_recipe['label'] : false;
					$f_type 		= ! empty( $field_recipe['type'] ) ? $field_recipe['type'] : '';
					$wrap_class		= ! empty( $field_recipe['wrap_class'] ) ? $field_recipe['wrap_class'] : '';

					if ( $repeater ) {
						$f_name 	= $field_id . '[' . $i . '][' . $f_id . ']';
						$f_val 		= ! empty( $saved_value[$i][$f_id] ) ? $saved_value[$i][$f_id] : $f_default;
					} else {
						$f_name 	= $field_id . '[' . $f_id . ']';
						$f_val 		= ! empty( $saved_value[$f_id] ) ? $saved_value[$f_id] : $f_default;
					}

					echo '<div class="fupi_r3_field fupi_field_type_' . $f_type . ' fupi_field_' . $f_id . '_wrap '.$wrap_class.'">';

						if ( ! empty( $f_label ) && $f_type != 'checkbox' ) {
							if ( $f_type == 'r3' ) {
								echo '<p class="fupi_r3_section_label">' . esc_attr($f_label) . '</p>';
							} else {
								$label_output = '<label class="fupi_r3_field_label">' . esc_attr($f_label);

								// POPUP

								if( isset( $field_recipe['label_popup'] ) ){
									$label_output .= '<button type="button" class="fupi_open_popup_i fupi_open_popup">i</button><div class="fupi_popup_content">'.$field_recipe['label_popup'].'</div>';
								}

								// POPUP 2 - with a warning

								if( isset( $field_recipe['label_popup2'] ) ){
									$label_output  .= '<button type="button" class="fupi_open_popup_i fupi_open_popup fupi_popup_warning">i</button><div class="fupi_popup_content">'.$field_recipe['label_popup2'].'</div>';
								}

								echo $label_output . '</label>';
							}
						}

						if ( $f_type != 'label' ) $this->fupi_field_html($field_recipe, $f_name, $f_val);

					echo '</div>';
				}

				if ( $repeater ){

					echo '<div class="fupi_r3_btns '. $btns_class .'">
						<button type="button" class="fupi_r3_btn fupi_btn_remove dashicons dashicons-minus"></button>
						<button type="button" class="fupi_r3_btn fupi_btn_add dashicons dashicons-plus-alt2"></button>
					</div>';
				}

			echo '</div>'; // r3 section end
		}

		if ( ! empty( $output ) ) echo $output;

	break;

	// =======

	case 'text':
	case 'number':
	case 'url':
	case 'email':
	case 'password': // If it is a text field
		$disabled = ! empty ( $recipe['disabled'] ) ? ' disabled="' . esc_attr( $recipe['disabled'] ) . '"' : '';

		if ( $recipe['type'] == 'number' ) printf('<button type="button" class="fupi_number_decrease"><span class="dashicons dashicons-remove"></span></button>');
		printf( '<input type="'.$recipe['type'].'" name="%1$s" id="%1$s" placeholder="%2$s" value="%3$s" class="%4$s" data-target="%5$s" %6$s %7$s/>', $field_id, $placeholder, esc_attr($saved_value), $el_class, $el_data_target, $disabled, $data_format );
		if ( $recipe['type'] == 'number' ) printf('<button type="button" class="fupi_number_increase"><span class="dashicons dashicons-insert"></span></button>');
	break;

	// =======

	case 'toggle':
		$el_data_name = ! empty ( $recipe['el_data_name'] ) ? $recipe['el_data_name'] : '';
		printf( '<label class="fupi_switch"><input type="checkbox" name="%1$s" id="%1$s" value="1" %2$s class="%3$s" data-target="%4$s" data-name="%5$s"/><span class="fupi_slider"></span></label>', $field_id, checked(1, $saved_value, false), $el_class, $el_data_target, $el_data_name );
	break;

	// =======

	case 'checkbox':
		if ( $is_r3 ) echo '<label>';
		printf( '<input type="checkbox" name="%1$s" id="%1$s" value="1" %2$s class="%3$s" data-target="%4$s"/>', $field_id, checked(1, $saved_value, false), $el_class, $el_data_target );
		if ( $is_r3 ) echo $recipe['label'] . '</label>';
	break;

	// =======

	case 'multi checkbox':
		if( ! empty ( $recipe['options'] ) && is_array( $recipe['options'] ) ){
			foreach( $recipe['options'] as $key => $label ) {

				$checked = false;

				if ( is_array( $saved_value ) ) {
					if ( in_array( $key, $saved_value) ){
						$checked = true;
					}
				} else if ( $saved_value == $key ) {
					$checked = true;
				}

				$option_data_target= '';
				if ( ! empty ( $recipe['options_data_targets'] ) && ! empty ( $recipe['options_data_targets'][$key] ) ) {
					$option_data_target = $recipe['options_data_targets'][$key];
				}

				printf(
					'<label><input type="checkbox" name="%1$s[]" value="%2$s" id="%1$s_%2$s" %4$s class="%5$s" data-target="%6$s">%3$s</label><div class="fupi_spacer"></div>',
					$field_id,
					$key,
					$label,
					checked( $checked, true, false ),
					$el_class,
					$option_data_target
				);
			}
			break;
		} else { break; }
	break;

	// =======

	case 'roles multi checkbox':

		global $wp_roles;
		$roles = $wp_roles->roles;

		foreach( $roles as $key => $data_arr ) {

			$checked = false;

			if ( is_array( $saved_value ) ) {
				if ( in_array( $key, $saved_value) ){
					$checked = true;
				}
			} else if ( $saved_value == $key ) {
				$checked = true;
			}

			printf(
				'<label><input type="checkbox" name="%1$s[]" value="%2$s" id="%1$s_%2$s" class="%5$s" %4$s>%3$s</label><div class="fupi_spacer"></div>',
				$field_id,
				$key,
				$data_arr['name'],
				checked( $checked, true, false ),
				$el_class
			);
		};

	break;

	// =======

	case 'taxonomies multi checkbox':

		$taxonomies = get_taxonomies();
		$fupi_main = get_option('fupi_main');

		foreach( $taxonomies as $tax_slug ){

			if ( $tax_slug == 'fupi_page_labels' || $tax_slug == 'category' || $tax_slug == 'post_tag' || $tax_slug == 'post_format' ) continue;

			$checked = false;

			if ( is_array( $saved_value ) ) {
				if ( in_array( $tax_slug, $saved_value) ){
					$checked = true;
				}
			} else if ( $saved_value == $tax_slug ) {
				$checked = true;
			}

			printf(
				'<label><input type="checkbox" name="%1$s[]" value="%2$s" id="%1$s_%2$s" %3$s class="%4$s">%2$s</label><div class="fupi_spacer"></div>',
				$field_id,
				$tax_slug,
				checked( $checked, true, false ),
				$el_class
			);
		};

	break;

	// =======

	case 'textarea': // If it is a textarea

		if ( ! empty( $recipe['format'] ) ) {
			if ( $recipe['format'] == 'htmlentities' ) {
				$val = esc_textarea( html_entity_decode( $saved_value, ENT_QUOTES ) );
			} else {
				$val = esc_textarea($saved_value);
			}
		} else {
			$val = esc_textarea($saved_value);
		}
		printf( '<textarea name="%1$s" id="%1$s" placeholder="%2$s" rows="5" cols="50" class="%4$s" data-target="%5$s" %6$s>%3$s</textarea>', $field_id, $placeholder, $val , $el_class, $el_data_target, $data_format );
	break;

	// =======

	case 'button': // If it is a textarea
		$button_text 		= ! empty ( $recipe['button_text'] ) ? esc_attr( $recipe['button_text'] ) : '';
		$icon		 		= ! empty ( $recipe['icon'] ) ? '<span class="' . esc_attr( $recipe['icon'] ) . '"></span> ' : '';
		$href		 		= ! empty ( $recipe['href'] ) ? esc_attr( $recipe['href'] ) : '';
		$target		 		= ! empty ( $recipe['target'] ) ? 'target="' . esc_attr( $recipe['target'] ) . '"' : '';

		if ( ! empty( $href ) ) {
			printf( '<a href="%1$s" %2$s class="%3$s">%4$s%5$s</a>', $href, $target, $el_class, $icon, $button_text );
		} else {
			printf( '<button type="button" class="%1$s">%2$s%3$s</button>', $el_class, $icon, $button_text );
		}

	break;

	// =======

	case 'select': // If it is a select dropdown
		if( ! empty ( $recipe['options'] ) && is_array( $recipe['options'] ) ){
			$options_markup = '';

			foreach( $recipe['options'] as $key => $label ){
				$options_markup .= sprintf( '<option value="%s" %s>%s</option>', $key, selected( $saved_value, $key, false ), $label );
			}

			printf( '<select name="%1$s" id="%1$s" class="%2$s" data-target="%3$s">%4$s</select>', $field_id, $el_class, $el_data_target, $options_markup );
		}
	break;

	// =======

	case 'taxonomies select':
		$taxonomies = get_taxonomies();

		$options_markup = '<option value=\'\'>' . esc_html__('None', 'fupi') . '</option>';

		foreach( $taxonomies as $tax_slug ){
			$options_markup .= sprintf( '<option value="%s" %s>%s</option>', $tax_slug, selected( $saved_value, $tax_slug, false ), $tax_slug );
		}

		printf( '<select name="%1$s" id="%1$s" class="%2$s" data-target="%3$s">%4$s</select>', $field_id, $el_class, $el_data_target, $options_markup );

	break;

	// =======

	case 'select2':
		if( ! empty ( $recipe['options'] ) && is_array( $recipe['options'] ) ){

			$multiple = empty( $recipe['multiple'] ) ? '' : 'multiple="multiple"';
			$save_as_multiple = empty( $recipe['multiple'] ) ? '' : '[]';

			wp_enqueue_style( 'fupi-select2-css', 'https://cdn.jsdelivr.net/npm/select2@4.1.0-rc.0/dist/css/select2.min.css', array(), '4.1.0-rc.0');
			wp_enqueue_script( 'fupi-select2-js', 'https://cdn.jsdelivr.net/npm/select2@4.1.0-rc.0/dist/js/select2.min.js', array('jquery', 'fupi-admin-helpers-js'), '4.1.0-rc.0');

			$output = '<select name="'.$field_id. $save_as_multiple.'" class="'.$el_class.'" ' . $multiple . '>';

			foreach( $recipe['options'] as $key => $label ){

				// Get html that marks option as selected

				$selected_html = '';

				if ( empty( $recipe['multiple'] ) ){
					$selected_html = selected( $saved_value, $key, false );
				// multi-select keeps values in an array
				} else {
					if ( ! empty( $saved_value ) && is_array( $saved_value ) ){
						foreach ( $saved_value as $val ) {
							if ( $val == $key ) {
								$selected_html = 'selected="selected"';
								break;
							}
						}
					}
				}

				$output .= '<option value="'.$key.'" '. $selected_html .'>'.$label.'</option>';
			}

			$output .= '</select>';
			echo $output;
		}
	break;

	// =======

	case 'radio':
		if( ! empty ( $recipe['options'] ) && is_array( $recipe['options'] ) ){
			$options_markup = '';
			foreach( $recipe['options'] as $key => $label ) {

				$option_data_target= '';
				if ( ! empty ( $recipe['options_data_targets'] ) && ! empty ( $recipe['options_data_targets'][$key] ) ) {
					$option_data_target = $recipe['options_data_targets'][$key];
				}

				$checked = $saved_value == $key ? true : false;

				printf(
					'<label><input type="radio" name="%1$s" value="%2$s" id="%1$s_%2$s" %4$s class="%5$s" data-target="%6$s">%3$s</label><div class="fupi_spacer"></div>',
					$field_id,
					$key,
					$label,
					checked( $checked, true, false ),
					$el_class,
					$option_data_target
				);
			}
			break;
		} else { break; }
	break;

	// =======

	case 'hidden':
		printf( '<input type="hidden" name="%1$s" id="%1$s" value="%2$s"/>', $field_id, $saved_value );
	break;

	// =======

	case 'empty':
	break;

	// ======= DEPRECATED ========

	case 'repeater':
		$el_data_field1_txt = ! empty ( $recipe['el_data_field1_txt'] ) ? $recipe['el_data_field1_txt'] : '';
		$el_data_field1_type = ! empty ( $recipe['el_data_field1_type'] ) ? $recipe['el_data_field1_type'] : 'selector';
		$el_data_field1_req = ! empty ( $recipe['el_data_field1_req'] ) ? $recipe['el_data_field1_req'] : '';
		$el_data_field1_err = ! empty ( $recipe['el_data_field1_err'] ) ? $recipe['el_data_field1_err'] : '';
		$el_data_field2_txt = ! empty ( $recipe['el_data_field2_txt'] ) ? $recipe['el_data_field2_txt'] : '';
		$el_data_field2_req = ! empty ( $recipe['el_data_field2_req'] ) ? $recipe['el_data_field2_req'] : '';
		$el_data_field2_err = ! empty ( $recipe['el_data_field2_err'] ) ? $recipe['el_data_field2_err'] : '';
		printf( '<textarea name="%1$s" id="%1$s" rows="5" cols="50" class="fupi_repeater %3$s" data-target="%4$s" data-field1_txt="%5$s" data-field1_req="%6$s" data-field1_err="%7$s" data-field2_txt="%8$s" data-field2_req="%9$s" data-field2_err="%10$s" data-field1_type="%11$s">%2$s</textarea>', $field_id, esc_textarea($saved_value), $el_class, $el_data_target, $el_data_field1_txt, $el_data_field1_req, $el_data_field1_err, $el_data_field2_txt, $el_data_field2_req, $el_data_field2_err, $el_data_field1_type );
	break;
}

// REQUIRED TEXT

if ( isset( $recipe['required'] ) ) {
	$req_txt = isset( $recipe['required text'] ) ? esc_attr( $recipe['required text'] ) : esc_html( 'This field is required', 'fupi' );
	echo '<span class="fupi_req_txt">' . $req_txt . '</span>';
}

// TOOLTIP

if( isset( $recipe['tooltip'] ) ){
	printf( '<div class="fupi_tooltip"><span class="dashicons dashicons-editor-help"></span><span class="fupi_tooltiptext">%s</span></div>', $recipe['tooltip'] );
}

// TEXT AFTER FIELD

if( isset( $recipe['after field'] ) ){
	printf( '<span class="fupi_after_field"> %s</span>', $recipe['after field'] );
}

// POPUP

if( isset( $recipe['popup'] ) ){
	printf( '<button type="button" class="fupi_open_popup_i fupi_open_popup">i</button><div class="fupi_popup_content">%s</div>', $recipe['popup'] );
}

// POPUP 2 - with a warning

if( isset( $recipe['popup2'] ) ){
	printf( '<button type="button" class="fupi_open_popup_i fupi_open_popup fupi_popup_warning">i</button><div class="fupi_popup_content">%s</div>', $recipe['popup2'] );
}

// TEXT BELOW FIELD

if( isset( $recipe['under field'] ) ){
	printf( '<div class="fupi_under_field">%s</div>', $recipe['under field'] );
}

?>
