// CREATE LINKS TO SECTIONS (IN SIDEBAR)

(() => {

	let fupi_settings_form 	= FP.findID('fupi_settings_form'),
		fupi_admin_page 	= FP.getUrlParamByName('page'),
		wp_menu_items 		= FP.findAll('#toplevel_page_full_picture_tools ul.wp-submenu a'),
		headlines 			= FP.findAll( '#fupi_settings_form h2' ),
		side_nav_wrap 		= FP.findID('fupi_side_menu');

	function add_menu_items(){

		let non_fupi_link 		= false,
			is_main 			= false;

		wp_menu_items.forEach( ( wp_menu_item, i ) => {

			let is_current_page = wp_menu_item.classList.contains('current'),
				output = '';

			// add <hr> above the first of the general settings page links
			if ( ! is_main && ( wp_menu_item.href.indexOf('_main') != -1 || wp_menu_item.href.indexOf('_woo') != -1 ) ) {
				is_main = true;
				output += '<hr>';
			}

			// add <hr> above the first freemius page link
			if ( wp_menu_item.href.indexOf('full_picture_tools-') != -1 && ! non_fupi_link ) {
				is_main = false; // this removes the addition of the "cog" icon later
				non_fupi_link = true;
				output += '<hr>';
			}

			// add submenu
			if ( is_current_page ) {
				output += '<div id="fupi_side_sub_nav">' + '<div id="fupi_nav_page_title">' + wp_menu_item.innerText + '</div>';

				if ( wp_menu_item.href.indexOf('tools') == -1 ) {
					output += add_sub_items();
				}

				output += '</div>';

			// add link to top page
			} else {
				output += '<a href="' + wp_menu_item.href + '" class="fupi_nav_page_link"><span>' + wp_menu_item.innerText + ' </span></a>';
			}

			output += '</div>';

			// line after "modules"
			if ( i == 0 ){
				output += '<hr>';
			}

			side_nav_wrap.insertAdjacentHTML('beforeend', output );

		})
	}

	function add_sub_items(){

		if ( headlines.length > 1 ) {

			let active_nav_tab = FP.findFirst('.nav-tab-active'),
				section_links = '',
				active_tab = fupi_settings_form.dataset.activetab,
				is_pro_code = ! FP.findID('fupi_show_pro');

			headlines.forEach( ( headline, i ) => {

				let h_txt = headline.innerText,
					current_class = i == 0  ? 'active' : '',
					show = true;

				// hide sections marked with "*" when they have no fields
				if ( h_txt.indexOf('*') != -1 ){

					// remove "*" from section\'s title
					h_txt = h_txt.replace('*','');
					headline.innerText = h_txt;

					// check if the section has no fields
					show = ( headline.nextElementSibling && headline.nextElementSibling.tagName == 'TABLE' ) || ( headline.nextElementSibling.nextElementSibling && headline.nextElementSibling.nextElementSibling.tagName == 'TABLE' );
				}

				if ( show ) {
					section_links += '<button type="button" data-target="hook_' + active_tab + '_' + i + '" class="fupi_nav_section_btn ' + current_class + '"><span>' + h_txt + '</span></button>';
					headline.setAttribute( 'id', 'hook_' + active_tab + '_' + i );
					headline.classList.add( 'fupi_hook', 'fupi_el');
				}

			});

			return section_links;

		} else {
			return '';
		}
	}

	function enable_sections_toggle(){

		// headlines = FP.findAll('#fupi_settings_form h2');
		let fupi_page_nav_links = FP.findAll('#fupi_page_nav .fupi_nav_section_btn:not(.fupi_never_active)');

		// make 1st section visible & unhide form
		if ( headlines.length > 1 ) show_section( headlines[0].id ); // show first section
		// if ( fupi_settings_form ) fupi_settings_form.classList.remove('fupi_hidden');

		// add events to links that add a target to url & show sections
		if ( fupi_page_nav_links.length > 1 ) {

			fupi_page_nav_links.forEach( link => {
				link.addEventListener( 'click', () => {
					if ( ! link.classList.contains('active') ) {
						FP.findFirst('#fupi_page_nav .fupi_nav_section_btn.active').classList.remove('active');
						let hook = link.dataset.target;
						show_section( hook );
					}
				} )
			} )
		}
	}

	function show_section( hook ) {

		let section_elements 	= FP.findAll( '.fupi_el, .form-table' ),
			pseudo_link 		= FP.findFirst( '.fupi_nav_section_btn[data-target="' + hook + '"]' );

		// hide all sections
		section_elements.forEach( element => element.classList.add('fupi_hidden'));

		// show chosen section
		FP.findAll('#' + hook + ', #' + hook + '+*, #' + hook + '+*+table').forEach( element => element.classList.remove('fupi_hidden'));

		// add class to current pseudo link
		pseudo_link.classList.add('active');
	}

	if ( fupi_admin_page && fupi_admin_page != 'full_picture_tools' && fupi_settings_form) {
		add_menu_items();
		enable_sections_toggle();
	};

})();

(() => {

	// DISABLE SETTINGS NOT AVAILABLE FOR USERS ON CERTAIN PLANS

	let fupi_content_el = FP.findID('fupi_content');

	if ( fupi_content_el ){

		let settings_fields = [],
			settings_descr = [];

		switch ( fupi_content_el.dataset.licence ) {

			// disable all premium settings for FREE
			case 'free':
				settings_fields = FP.findAll('.fupi_lite input, .fupi_lite textarea, .fupi_lite select, .fupi_pro input, .fupi_pro textarea, .fupi_pro select');
				settings_descr = FP.findAll('.fupi_section_descr.fupi_lite, .fupi_section_descr.fupi_pro');
			break;

			// disable PRO settings for LITE
			case 'lite':
				settings_fields = FP.findAll('.fupi_pro input, .fupi_pro textarea, .fupi_pro select, .fupi_section_descr.fupi_pro');
				settings_descr = FP.findAll('.fupi_section_descr.fupi_pro');
			break;
		}

		if ( settings_fields.length == 0 ){
			let pro_switch = FP.findID('fupi_show_pro');
			if ( pro_switch ) pro_switch.style.display = "none";
		} else {
			settings_fields.forEach( field => {
				let wrap = field.closest('tr');

				field.disabled = true;
				wrap.classList.remove('fupi_pro');
				wrap.classList.remove('fupi_lite');
				wrap.classList.add('fupi_premium');
				wrap.classList.add('fupi_disabled');
			});
		}

		settings_descr.forEach( descr => {
			descr.classList.remove('fupi_lite');
			descr.classList.remove('fupi_pro');
			descr.classList.add('fupi_premium');
		} )
	}
})();

(() => {

	// CONDITIONS
	// toggle fields that can be conditionally toggled

	let condition_fields = FP.findAll( '.fupi_condition:not(.fupi_premium)' ),
		fupi_content_el = FP.findID('fupi_content');

	if ( condition_fields.length > 0 ) {

		condition_fields.forEach( field => {

			// after pageload
			sync_settings( field );

			// after click
			field.onclick = (e) => { sync_settings( field ); };

			// on key up
			field.onkeyup = (e) => { sync_settings( field ); }
		})
	};


	function sync_settings( field ) {

		let field_type = field.getAttribute('type') || field.tagName.toLowerCase(),
			table_rows = FP.findAll('.' + field.dataset.target),
			val = false;

		switch (field_type) {

			case 'checkbox':
				toggle_rows(table_rows, field.checked);
			break;

			case 'number':
				val = field.value != 0 ? true : false;
				toggle_rows(table_rows, val);
			break;

			case 'text':
			case 'textarea':
				val = !!field.value;
				toggle_rows(table_rows, val);
			break;

			case 'select' :
				val = field.value;
				toggle_rows(table_rows, val, 'select');
			break;

		}
	}

	function toggle_rows( table_rows, val, field_type ) {

		table_rows.forEach( row => {

			if ( ! row.classList.contains('fupi_premium') ) {

				let inputs = FP.findAll( 'input, textarea, select', row );

				if ( field_type != 'select' ) {

					let enabled = val;
					enabled ? row.classList.remove( 'fupi_disabled' ) : row.classList.add( 'fupi_disabled' ); // toggle class on table row

					// toggle state of inputs
					if ( inputs.length > 0) {
						inputs.forEach( input => {
							input.disabled = ! enabled;
						})
					}

				} else {

					if ( row.classList.contains('fupi_cond_val_' + val ) ) {
						row.classList.remove( 'fupi_disabled' );
						inputs.forEach( input => input.disabled = false )
					} else {
						row.classList.add( 'fupi_disabled' );
						inputs.forEach( input => input.disabled = true )
					}
				}
			}
		})
	}

})();

//

(() => {

	let fupi_settings_form 	= FP.findID('fupi_settings_form'),
		fupi_admin_page 	= FP.getUrlParamByName('page'),
		wp_menu_items 		= FP.findFirst('#toplevel_page_full_picture_tools ul'),
		content_el 			= FP.findID('fupi_content');

	function mark_tables_with_classes() {

		let tables = FP.findAll('#fupi_settings_form>table.form-table');

		tables.forEach( ( table, i ) =>{
			let table_class = 'fupi_tools_list';
			switch (i) {
				case 0:
					table.classList.add( 'fupi_table_grid' );
				break;
				case 1:
					table_class = 'fupi_privacy_list';
				break;
				case 2:
					table_class = 'fupi_ext_list';
				break;
			}
			table.classList.add( table_class );
		})
	}

	function tools_enabled(){
		return FP.findAll('.fupi_tools_list tr:not(.fupi_disabled) input[type="checkbox"]:checked').length > 0;
	}

	function show_hide_next_btn_on_tools_toggle(){
		document.addEventListener('change', e=>{
			show_or_hide_next_btn();
		});
	}

	function show_or_hide_next_btn(){
		let checkboxes = FP.findAll('.fupi_tools_list tr:not(.fupi_disabled) input[type="checkbox"]');

		if ( checkboxes.some(chk => chk.checked) ){
			content_el.classList.add('fupi_has_toggled_tools');
		} else {
			content_el.classList.remove('fupi_has_toggled_tools');
		}
	}

	function listen_to_clicks_on_buttons(){

		// CANCEL
		FP.findID('fupi_cancel_btn').addEventListener('click', ()=>{ change_step('list'); })

		// ADD TOOLS
		FP.findID('fupi_add_tools_btn').addEventListener('click', ()=>{ change_step('first'); })

		// NEXT
		FP.findID('fupi_next_btn').addEventListener('click', ()=>{ change_step('next'); });

		// PREV
		FP.findID('fupi_prev_btn').addEventListener('click', ()=>{ change_step('prev'); });

		document.body.addEventListener('click', e=>{
			let target = e.target;

			if ( target.parentElement.classList.contains('fupi_change_modules_link') ) target = target.parentElement;

			if ( target.classList.contains('fupi_change_modules_link') ) {
				switch (target.id) {
					case 'fupi_change_tools':
						show_step(11);
					break;
					case 'fupi_change_privacy':
						show_step(22);
					break;
					case 'fupi_change_ext':
						show_step(33);
					break;
				}
			}
		})
	}

	function change_step( step ){

		// get current step number
		let curr_step = parseInt( content_el.dataset.step ),
			next_step = curr_step;

		// get what step to show
		if ( step == 'list' ) next_step = 0; // screen with enabled modules
		if ( step == 'first' ) next_step = 1;
		if ( step == 'next' ) ++next_step;
		if ( step == 'prev' ) --next_step;

		show_step(next_step);
	}

	function show_step(step){
		content_el.dataset.step = step;
		show_or_hide_next_btn();
		window.scrollTo({top: 0, behavior: 'smooth'});
	}

	function generate_list_of_enabled_modules(){

		let has_ext = false,
			has_privacy = false,
			has_tools = false;

		['tools', 'privacy', 'ext'].forEach( module_type => {


			let enabled_modules_wrap = FP.findFirst('#fupi_' + module_type + '_faux tbody'),
				enabled_modules = FP.findAll('.fupi_' + module_type + '_list tr:not(.fupi_disabled) input[type="checkbox"]:checked');

			enabled_modules.forEach( module => {

				let tr_el = module.closest('tr').cloneNode(true)
				 	label = FP.findFirst('label', tr_el),
					module_name = FP.findFirst('input', label).name,
					module_id = module_name.replace('fupi_tools[',''),
					after_field_info = FP.findFirst('.fupi_after_field', tr_el);

				module_id = module_id.replace(']', '');

				let menu_link = FP.findFirst( 'a[href*=full_picture_' + module_id + ']', wp_menu_items);

				if ( menu_link ){

					let new_link = document.createElement('a');

					new_link.href = menu_link.href;
					new_link.classList.add('button', 'button-secondary'),
					new_link.textContent = 'Settings';

					label.parentElement.insertBefore(new_link, label)
				} else {

					let no_link_info = document.createElement('span');

					no_link_info.textContent = 'This module has no settings page'
					no_link_info.classList.add('fupi_no_link_info');

					label.parentElement.insertBefore(no_link_info, label);
				}

				if ( label ) FP.remove( label );
				if ( after_field_info ) FP.remove( after_field_info );

				enabled_modules_wrap.appendChild(tr_el);

				if ( module_type == 'tools' ) has_tools = true;
				if ( module_type == 'privacy' ) has_privacy = true;
				if ( module_type == 'ext' ) has_ext = true;
			});
		} );

		// if there are any enabled ext or privacy modules hide the "no modules enabled" notices
		if ( has_privacy ) FP.findID('fupi_no_privacy_modules_selected').style.display = 'none';
		if ( has_ext ) FP.findID('fupi_no_modules_selected').style.display = 'none';

		// and finally show the whole generated list
		content_el.classList.add('fupi_has_enabled_tools');
	}


	if ( fupi_admin_page && fupi_admin_page == 'full_picture_tools' && fupi_settings_form) {

		mark_tables_with_classes();
		listen_to_clicks_on_buttons();

		if ( ! tools_enabled() ) {
			change_step('first');
		} else {
			generate_list_of_enabled_modules();
		}

		show_hide_next_btn_on_tools_toggle();
	};

})();

;(()=>{
	/*
	// shows notifications about possible issues with certain location choices in cookie notice settings

	let show_in_country = FP.findID('fupi_cook[show_notice]'),
		require_in_country = FP.findID('fupi_cook[enable_scripts_after]');

	function display_warning( message ){
		 [require_in_country].forEach( select => {
		 	let wrap = select.closest('td'),
				warning = FP.findFirst( '.fupi_cookie_country_warning', wrap );

			if ( warning ){
				if ( ! message ){
					FP.remove( warning );
				} else {
					warning.textContent = message;
				}
			} else {
				if ( message ) wrap.insertAdjacentHTML( 'beforeend', '<p class="fupi_cookie_country_warning">' + message + '</p>' );
			}

		 });
	};

	function show_warnings(){
		if ( show_in_country.value == 'in_countries' ){
			switch (require_in_country.value) {
				case 'consent':
					display_warning('Attention! You chose to show cookie notice only in specific countries but require cookie consent in ALL countries. This means that visitors from countries where chookie notice doesn\'t show won\'t have a chance to agree to cookies and will NEVER be tracked.');
				break;
				case 'auto':
					display_warning('Attention! If the list of countries in which you show cookie notice does not contain countries where cookie consent is required for tracking some of your visitors will NEVER be tracked.');
				break;
				case 'choice':
					display_warning('Attention! You chose to show cookie notice and require consent before tracking in specific countries. Please make sure that the second list of countries contains only countries listed in the first one. Otherwise some visitors will never see the notice and won\'t be tracked.');
				break;
				default:
					display_warning(false);
			}
		} else {
			display_warning(false);
		}
	}


	if ( show_in_country && require_in_country ) {

		show_warnings();

		show_in_country.addEventListener( 'change', e=>{
			show_warnings();
		} )

		require_in_country.addEventListener( 'change', e=>{
			show_warnings();
		} )
	}
	*/

	function remove_errors(){

		FP.findAll('.fupi_error_notice.fupi_active').forEach( err_notice => {
			err_notice.classList.remove('fupi_active');
		} );

		FP.findAll('.fupi_field_error').forEach( sel => {
			sel.classList.remove('fupi_field_error');
		} )
	}

	function check_countries_for_duplicates( once ){

		let optin_field = FP.findID( 'fupi_cook[optin_countries]' ),
			optout_field = FP.findID( 'fupi_cook[optout_countries]' );

		[optin_field, optout_field].forEach( field =>{

			field.addEventListener('keyup', e => {

				let in_str = optin_field.value;
				in_str = in_str.replace(' ', '');
				let in_arr = in_str.split(',');


				let out_str = optout_field.value;
				out_str = out_str.replace(' ', '');
				let out_arr = out_str.split(',');

				let arrays_have_common_elements = in_arr.filter(in_val => out_arr.includes(in_val));
				console.log(in_arr, out_arr, arrays_have_common_elements);

				if ( arrays_have_common_elements.length > 0 ) {

					field.classList.add('fupi_field_error');

					let err_notice = FP.findFirst( '.fupi_error_notice', field.closest('tr') );
					if ( err_notice ) err_notice.classList.add('fupi_active');

				} else {
					remove_errors();
				}

			})
		})
	};

	let optin_select = FP.findID('fupi_cook[optin]');

	if ( optin_select ){

		let optout_select = FP.findID('fupi_cook[optout]');

		// check for errors when people select from dropdowns
		[optin_select, optout_select].forEach( sel => {

			sel.addEventListener('change', ()=>{

				remove_errors();

				if ( optin_select.value == 'all' && optout_select.value == 'specific' ) {

					let notice = FP.findFirst('.fupi_countries_error', sel.closest('tr'));

					notice.classList.add('fupi_active');
					sel.classList.add('fupi_field_error');

				} else if ( optin_select.value == 'none' && optout_select.value == 'none' ) {

					let notice = FP.findFirst('.fupi_countries_error_3', sel.closest('tr'));
					
					notice.classList.add('fupi_active');
					sel.classList.add('fupi_field_error');

				} else if ( optin_select.value == 'specific' && optout_select.value == 'specific' ) {

					check_countries_for_duplicates(true);

				}
			})

		})

		// Check for errors when people enter countries in the list
		check_countries_for_duplicates();
	}
})();

(()=>{

	// TOGGLE VISIBILITY OF PREMIUM SETTINGS

	let show_pro_switch = FP.findID('fupi_show_pro_switch'),
		fupi_main_wrap = FP.findID('fupi_main');

	function hideSideLink(h){
		var side_link = FP.findFirst('.fupi_nav_section_btn[data-target="' + h.id + '"]');
		side_link.classList.add('fupi_hidden');
	}

	function hide_premium_settings(){

		FP.deleteCookie( 'fupi_show_pro' );

		// hide all premium settings with CSS
		fupi_main_wrap.classList.add('fupi_pro_hide');

		// hide settings tables, descriptions and side nav links

		// get headings of all settings sections
		let section_headings = FP.findAll('#fupi_settings_form h2.fupi_hook'),
			active_is_closed = false;

		// for each heading
		section_headings.forEach( h => {


			let descr = h.nextElementSibling, // always exists since it contains the h3 with section title
				table = descr.nextElementSibling.classList.contains('form-table') ? descr.nextElementSibling : false,
				is_all_premium = descr.classList.contains('fupi_premium'),
				is_descr_hidden = descr.classList.contains('fupi_hidden');

			// if the descr is marked as premium then all the content is in the section is premium
			if ( is_all_premium ) {
				hideSideLink(h);
				if ( ! is_descr_hidden ) active_is_closed = true; // mark that the content that the user was looking at was closed
				descr.classList.add('fupi_hidden');
				if (table) table.classList.add('fupi_hidden');
			} else {
				if (table) {
					// get all non-premium fields
					let not_premium_tr = FP.findAll('tr:not(.fupi_premium)', table);
					// if we have no such fields
					if ( not_premium_tr.length == 0 ) {
						hideSideLink(h);
						if ( ! is_descr_hidden ) active_is_closed = true;  // mark that the content that the user was looking at was closed
						descr.classList.add('fupi_hidden');
						if (table) table.classList.add('fupi_hidden');
					}
				}
			}
		})

		// if the section that the user was looking at was hidden
		if ( active_is_closed ) {

			// get first heading's id
			let hook = section_headings[0].id;

			// hide active from the side link
			FP.findFirst('.fupi_nav_section_btn.active').classList.remove('active');

			// make the first side link active
			FP.findFirst('.fupi_nav_section_btn').classList.add('active');

			// show content of the first section
			FP.findAll('#' + hook + ', #' + hook + '+*, #' + hook + '+*+table').forEach( element => element.classList.remove('fupi_hidden'));
		}
	}

	function show_premium_settings(){

		FP.setCookie( 'fupi_show_pro', true, 365 );

		let disabled_sidelinks = FP.findAll('.fupi_nav_section_btn.fupi_hidden');

		disabled_sidelinks.forEach( section => section.classList.remove('fupi_hidden') );
		fupi_main_wrap.classList.remove('fupi_pro_hide');
	}

	if ( show_pro_switch ) {

		// toggle PRO features after click
		show_pro_switch.addEventListener( 'change', e => e.target.checked ? show_premium_settings() : hide_premium_settings() )

		// toggle PRO features after load
		FP.readCookie('fupi_show_pro') ? show_pro_switch.click() : hide_premium_settings();
	}

})();

// FILTER THE LIST OF TOOLS

(() => {

	function add_event_to_filter() {

		// FITLER TOOLS BY TYPE

		let select_filter = FP.findFirst('#fupi_tools_filter select');

		select_filter.addEventListener('change', e => {

			let choice = e.target.value,
				tools = FP.findAll('.fupi_tools_list .fupi_tag');

			if ( !!choice ){

				let chosen_tools = FP.findAll('.fupi_tools_list .'+ choice);

				tools.forEach( tool => {
					tool.style.display = "none";
				});

				chosen_tools.forEach( chosen_tool => {
					chosen_tool.style.display = "table-row";
				})
			} else {
				tools.forEach( tool => {
					tool.style.display = "table-row";
				})
			}
		});

		// FILTER TOOLS BY COOKIE USAGE

		let nocookie_switch = FP.findID('fupi_show_nocookie_tools_switch');

		// toggle tools after click

		nocookie_switch.addEventListener( 'click', e => {

			if ( e.target.checked ) {
				document.body.classList.add('fupi_cookie_tools_hidden');
			} else {
				document.body.classList.remove('fupi_cookie_tools_hidden');
			}
		})

		// FILTER FREE TOOLS

		let freetols_switch = FP.findID('fupi_show_free_tools_switch');

		// toggle tools after click

		freetols_switch.addEventListener( 'click', e => {

			if ( e.target.checked ) {
				document.body.classList.add('fupi_free_tools_hidden');
			} else {
				document.body.classList.remove('fupi_free_tools_hidden');
			}
		})

		// FILTER TOOLS BY DASHBOARD STATS

		let admindash_switch = FP.findID('fupi_show_admindash_tools_switch');

		// toggle tools after click

		admindash_switch.addEventListener( 'click', e => {

			if ( e.target.checked ) {
				document.body.classList.add('fupi_admindash_tools_hidden');
			} else {
				document.body.classList.remove('fupi_admindash_tools_hidden');
			}
		})
	}

	if ( FP.findID('fupi_tools_filter') ) {
		add_event_to_filter();
	}
})();

// REPEATER

(() => {

	let textareas = FP.findAll('.fupi_repeater'),
		template = '<div class="fupi_repeater_line_wrap fupi_repeater_template">'+
		 	'<div class="fupi_repeater_fields_wrap fupi_ok">' +
				'<div class="fupi_repeater_p1_wrap">' +
					'<input class="fupi_repeater_field fupi_repeater_p1 fupi_required" type="text">' +
					'<span class="fupi_error_msg fupi_error_msg_p1" style="display: none;"></span>' +
					'<span class="fupi_required_msg fupi_required_msg_p1" style="display: none;"></span>' +
				'</div>' +
				'<div class="fupi_repeater_p2_wrap">' +
					'<input class="fupi_repeater_field fupi_repeater_p2" type="text">' +
					'<span class="fupi_error_msg fupi_error_msg_p2" style="display: none;"></span>' +
					'<span class="fupi_required_msg fupi_required_msg_p2" style="display: none;"></span>' +
				'</div>' +
			'</div>'+
			'<div class="fupi_repeater_btns_wrap">' +
				'<button type="button" class="fupi_repeater_btn fupi_btn_remove dashicons dashicons-minus"></button>' +
				'<button type="button" class="fupi_repeater_btn fupi_btn_add dashicons dashicons-plus-alt2"></button>' +
			'</div>' +
		'</div>';

	function add_repeater_fields( textarea, placement, line_content, is_disabled ) {

		// Add template to page
		placement.insertAdjacentHTML('afterend', template);

		let content = line_content ? line_content.split('@') : ['',''],
			addedTemplate = FP.findFirst('.fupi_repeater_template'),
			field1 = FP.findFirst('.fupi_repeater_p1', addedTemplate),
			field2 = FP.findFirst('.fupi_repeater_p2', addedTemplate),
			field2_req_text = textarea.dataset.field2_req;

		// Fill it with data

		// field 1
		field1.value = content[0];
		field1.disabled = is_disabled;
		field1.placeholder = textarea.dataset.field1_txt;
		field1.dataset.type = textarea.dataset.field1_type;
		FP.findFirst( '.fupi_error_msg_p1', addedTemplate ).textContent = textarea.dataset.field1_err;
		FP.findFirst( '.fupi_required_msg_p1', addedTemplate ).textContent = textarea.dataset.field1_req;

		// field 2
		field2.value = content[1] || '';
		field2.disabled = is_disabled;
		field2.placeholder = textarea.dataset.field2_txt;
		FP.findFirst( '.fupi_error_msg_p2', addedTemplate ).textContent = textarea.dataset.field2_err || '';
		if ( field2_req_text ) {
			field2.classList.add('fupi_required');
			FP.findFirst( '.fupi_required_msg_p2', addedTemplate ).textContent = field2_req_text;
		}

		// Remove template class
		addedTemplate.classList.remove('fupi_repeater_template');
	}

	function enable_line_buttons() {

		let form = FP.findID('fupi_settings_form');

		if ( form ) {
			form.addEventListener('click', e => {
				let target = e.target;
				if ( target.classList.contains('fupi_repeater_btn') ) {

					let btns_wrap = target.closest( '.fupi_repeater_line_wrap'),
						textarea = btns_wrap.previousElementSibling;

					if ( textarea && ! textarea.disabled ) {

						// remove line
						if ( target.classList.contains('fupi_btn_remove') ) {

							let fields_line = target.closest('.fupi_repeater_line_wrap'),
							repeater_section = target.closest('.fupi_repeater_section');

							FP.remove(fields_line);
							update_textarea(false, repeater_section);

						// add line
						} else {
							let fupi_repeater_section = target.closest('.fupi_repeater_section'),
							textarea = FP.findFirst('textarea', fupi_repeater_section),
							line_with_clicked_btn = target.closest('.fupi_repeater_line_wrap');

							add_repeater_fields(textarea, line_with_clicked_btn, false, false)
						}
					}
				}
			})
		}
	}

	function create_repeater_fields () {

		textareas.forEach( textarea => {

			let value = textarea.value,
				lines = value ? value.split(/\r?\n/) : [''], // split lines into array (if there are no lines create an empty array)
				lines_reversed = lines.reverse(),
				is_disabled = textarea.disabled;

			// create fields for each line
			lines_reversed.forEach( line => add_repeater_fields( textarea, textarea, line, is_disabled ) )

		} )
	}

	function is_selector_valid ( sel ) {
		try { document.querySelector( sel ) } catch (e) { return false; }
		return true;
	}

	function toggle_error_msg( field ) {

		let is_field_1 = field.classList.contains('fupi_repeater_p1'),
			error_msg = FP.findFirst( '.fupi_error_msg', field.parentElement );

		// for field 1
		if ( is_field_1 ) {

			// if field value is a CSS selector
			if ( field.dataset.type == 'selector' ) {

				if ( is_selector_valid( field.value ) ) {
					error_msg.style.display = 'none';
					field.classList.remove('fupi_error');
				} else {
					field.classList.add('fupi_error');
					error_msg.style.display = 'block';
				}
			}

		// if field 2
		} else {

			// if string contains characters from outside this list
			if ( /[^A-Za-z0-9 \-\_\[\]]/.test( field.value ) ) {
				field.classList.add('fupi_error');
				error_msg.style.display = 'block';
			} else {
				error_msg.style.display = 'none';
				field.classList.remove('fupi_error');
			}
		}
	}

	function toggle_required_msg( field ) {

		if ( field.classList.contains('fupi_required') ) {

			let req_msg = FP.findFirst( '.fupi_required_msg', field.parentElement );

			if ( field.value ) {
				field.classList.remove('fupi_empty');
				req_msg.style.display = 'none';
			} else {
				field.classList.add('fupi_empty');
				req_msg.style.display = 'block';
			}
		}
	}

	function mark_ok_lines( field ) {

		// check if all the required fields in this "line" are filled in correctly

		let fields_wrap = field.closest('.fupi_repeater_fields_wrap'),
		 	repeater_fields_with_errors = FP.findAll('.fupi_repeater_field.fupi_empty, .fupi_repeater_field.fupi_error', fields_wrap);

		repeater_fields_with_errors.length == 0 ? fields_wrap.classList.add('fupi_ok') : fields_wrap.classList.remove('fupi_ok');
	}

	function update_textarea( field, repeater_section = false) {

		// copy to textarea values of all correctly filled in fields

		let r_section = repeater_section || field.closest('.fupi_repeater_section'),
			field_wraps = FP.findAll('.fupi_ok', r_section), // get all fields from lines that have no erros
			textarea = FP.findFirst('textarea', r_section),
			new_val = '';

		if ( field_wraps.length > 0 ) {
			field_wraps.forEach( wrap => {

				let fields = FP.findAll('input', wrap).filter( input => !!input.value );

				if ( fields.length > 0 ) {
					if ( fields.length > 1 ) {
						new_val += fields[0].value.trim() + '@' + fields[1].value.trim()  + "\n";
					} else {
						new_val += fields[0].value.trim() + "\n";
					}
				}

			} )
		}

		textarea.value = new_val.trim();

		// trigger keyup event on textarea (for other functions to work properly)
		textarea.click();
	}

	// init
	if ( textareas ) {

		create_repeater_fields();
		enable_line_buttons();

		// when repeater fields lose focus...

		document.addEventListener('focusout', e => {

			let field = e.target;

			if ( field.classList && field.classList.contains('fupi_repeater_field') ){
				toggle_error_msg( field );
				toggle_required_msg( field );
				mark_ok_lines( field );
				update_textarea( field );
			}
		})
	}
})();

// SCRIPT REPEATER

(() => {

	function remove_or_clear_section(target){
		let closest_section_wrap = target.closest('.fupi_r3_section'),
			siblingSections = FP.getSiblings( closest_section_wrap );

		siblingSections = siblingSections.filter(sect => sect.classList.contains('fupi_r3_section')); // get only r3 sections

		// check if this is the only section on this level
		if ( siblingSections.length > 0 ) {
			FP.remove(closest_section_wrap);
		} else {
			clear_section(closest_section_wrap);
		}
	}

	function add_section(target){

		let current_section = target.closest('.fupi_r3_section'),
			cloned_section = current_section.cloneNode(true),
			scr_id_field = FP.findFirst('.fupi_r3_scr .fupi_field_id_wrap input', cloned_section);

		cloned_section = clear_section(cloned_section);
		cloned_section = remove_extra_repeaters(cloned_section);
		current_section.parentElement.insertBefore(cloned_section, current_section.nextSibling);

		if ( scr_id_field ) randomize_id( scr_id_field );
	}

	function destroy_all_select2s(){
		(($)=>{
			if ( $.isFunction($.fn.select2) ){
				$('.fupi_field_type_select2').each( (i, wrap) => {
					$wrap = $(wrap);
					if ( $wrap.hasClass('fupi_select2_enabled') ) {
						$wrap.find('select').select2('destroy');
						$wrap.removeClass('fupi_select2_enabled');
					}
				});
			}
		})(jQuery)
	}

	function enable_all_select2s(){
		jQuery(document).ready(function($) {
			if ( $.isFunction($.fn.select2) ){
				$('.fupi_field_type_select2').each( (i, wrap) => {
					$wrap = $(wrap);
					$wrap.find('select').select2();
					$wrap.addClass('fupi_select2_enabled');
				})
			}
		});
	}

	function rename_fields_in_section( sections_wrap, parent_sect_nums ){

		if ( ! parent_sect_nums ) parent_sect_nums = [];

		// console.log('sections_wrap', sections_wrap);

		// get all the sections that are direct descendants of the section wrap
		let sibling_sections = FP.nl2Arr( sections_wrap.children ),
			sections = sibling_sections ? sibling_sections.filter( siblig => siblig.classList.contains('fupi_r3_section') ) : [];

		// console.log('same level sections', sections);

		sections.forEach( (section, sect_i) => {

			let current_sect_nums = [...parent_sect_nums, sect_i];

			// get all the field wrappers tha are direct descendants of the section
			let section_children = FP.nl2Arr( section.children ),
				field_wrappers = section_children ? section_children.filter( child => child.classList.contains('fupi_r3_field') && ! child.classList.contains('fupi_field_type_label') ) : [];

			field_wrappers.forEach( ( field_wrap, field_i ) => {

				// if field wrapper contains a repeater run renaming function on its content
				if ( field_wrap.classList.contains('fupi_field_type_r3') ){

					rename_fields_in_section(field_wrap, current_sect_nums );

				// if the field wrapper contains form fields then modify their names and ids
				} else {

					let field = FP.findFirst('input, textarea, select', field_wrap),
						name_parts = field.name.split(/\]\[\d*?\]\[/g) || []; // this gives us strings with square brackets, like '][3]['. We don't need anything more than that. We use it only to count how many occurances of digits appear

					if (name_parts.length - 1 <= current_sect_nums.length ){ // there should always be 1 more name part than numbers to fill in

						// here we create a name and id
						let new_name = name_parts.shift(); // we take the first name part (this also lowers the number of elements in the array)
						name_parts.forEach( ( part, i ) => { new_name += '][' + current_sect_nums[i] + '][' + part; } );

						// console.log('new field name for ' + field.name + ' is ' + new_name);

						field.name = new_name;
						field.id = new_name;
					} else {
						console.error('name can\'t be constructed for ', field, ' with: ', name_parts, current_sect_nums );
					}
				};
			} );
		} );
	}

	function remove_extra_repeaters(section){
		let extra_section = FP.findFirst('.fupi_r3_repeater + .fupi_r3_repeater', section);
		if ( extra_section ) {
			FP.remove(extra_section);
			return remove_extra_repeaters(section);
		} else {
			return section;
		}
	}

	function clear_section(section){

		FP.findAll('textarea', section).forEach( area => area.value = '' );
		FP.findAll('input[type="checkbox"]', section).forEach( box => box.checked = false );
		FP.findAll('input[type="text"]', section).forEach( field => field.value = '' );
		FP.findAll('input[type="number"]', section).forEach( field => field.value = 0 );
		FP.findAll('input[type="password"]', section).forEach( field => field.value = '' );
		FP.findAll('select', section).forEach( sel => sel.value = FP.findFirst('option', sel).value );

		FP.findAll('.field_empty', section).forEach( field => field.classList.remove('field_empty') );

		return section;
	}

	function enable_section_buttons() {

		FP.findID('fupi_settings_form').addEventListener('click', e => {

			let btn = e.target;

			if ( btn.classList.contains('fupi_r3_btn') ){

				destroy_all_select2s();
				let section_wrap = btn.closest('td');

				if ( btn.classList.contains('fupi_btn_remove') ){
					remove_or_clear_section(btn);
					rename_fields_in_section(section_wrap);
				} else {
					add_section(btn);
					rename_fields_in_section(section_wrap);
				}

				enable_all_select2s();
			}
		})
	}

	function get_attributes_from_str( str ){

		// return an object with key-value pairs
		//
		// string example: 'data-param='abc def ghi' async src="https://www.googletagmanager.com/gtag/js?id=UA-11111111-1" data-param2="abc" id="anID" class="class1 class2"'


		let parts = str.trim().split(' '), // we break the string into parts by 'space' (spaces can occur in unexpected places! see example above)
			ok_parts = parts.filter( v => v ), // and remove empty values from array
			ret_val = {};

		for ( var i = 0; i < ok_parts.length; i++ ) {

			let part = ok_parts[i],
				eq_index = part.indexOf('='),
				last_char = part[part.length-1];

			// if we have an equal sign
			// (this means that the value before the equal sign is the key and the one after is the value)
			if ( eq_index != -1 ){

				// we check if the last element of the current part is an apostrophe. If it isn\'t then it means we split the value in two or more pieces and need to join it with the next string parts
				while ( ! ( last_char == '"' || last_char == "'" ) && i+1 <= ok_parts.length ) {
					part += ' ' + ok_parts[i+1];
					last_char = part[part.length-1];
					i++;
				}

				// if we have the key and the whole value
				if ( last_char == '"' || last_char == "'" ) {
					// we save the key and value pair
					let key_val = part.split(/=(.*)/s); // split on the first occurance of "=" sign. They may sometimes show up in values too
					ret_val[key_val[0]]= key_val[1];
				}

			// if we don't have an equal sign this is a single value
			} else {
				ret_val[part] = true;
			}
		}

		console.log('ret_val', ret_val);

		return ret_val;

	}

	function reformat_custom_scripts( script ){

		// remove all ending "</script>" tags
		script = script.replaceAll('</script>', '');

		// find all <script[something or nothing]> tags
		let start_regex = /<script[\s\S]*?>/gi,
			matches = script.match(start_regex),
			replacements = [],
			has_error = false;

		// get their attributes
		if ( Array.isArray( matches ) && matches.length > 0 ) matches.forEach( txt => {

			// remove "<script" and final ">"
			txt = txt.replace('<script','');
			txt = txt.slice(0, -1);

			if ( txt.length == 0 ) {

				replacements.push('');

			} else {

				// transform the "<script [attributes]>" into "getScript()"
				let attributes_obj = get_attributes_from_str(txt),
					src = attributes_obj.src ? attributes_obj.src : false,
					attrs_str = '';

				if ( ! src ) {
					console.error('Cannot re-format the <script> elements in the code. Missing "src" attribute.');
					has_error = true;
				}

				for (const [key, val] of Object.entries(attributes_obj)) {
					if ( key != 'src' ) attrs_str += '"' + key + '": ' + val + ', ';
				}

				replacements.push([src, attrs_str]);
			}
		} );

		if ( has_error ) return false;

		// prepare elements to replace <string [something or nothing]> in the original script
		replacements.forEach( ( repl, i ) =>{
			if ( repl ) replacements[i] = 'FP.getScript(' + repl[0] + ', false, {' + repl[1] + '});'; // repl[0] & repl[1] are already properly formatted
		} )

		// replace elements in string one by one

		let i = 0;

		return script.replace(start_regex, ()=>{ return replacements[i++]; });

		// e.target.value = e.target.value.replaceAll(/<.*?script[\s\S]*?>/g, '');
	}

	function enable_focusout_checks(){
		document.addEventListener( 'focusout', e => {

			switch (e.target.tagName) {
				case 'INPUT':
				case 'TEXTAREA':
				case 'SELECT':

					let r3_field_wrap = e.target.closest('.fupi_r3_field');

					if ( r3_field_wrap ) {

						// mark as empty
						e.target.value ? r3_field_wrap.classList.remove('field_empty') : r3_field_wrap.classList.add('field_empty');

						// sanitize script
						if ( e.target.tagName == 'TEXTAREA' ) {

							let new_value = reformat_custom_scripts( e.target.value ); // e.target.value = e.target.value.replaceAll(/<.*?script[\s\S]*?>/g, '');
							if ( new_value ) e.target.value = new_value;
						}

						// sanitize script id
						if ( FP.isIn( '[id]', e.target.name ) ) e.target.value = e.target.value.replace(/[^a-z0-9_]/g, "");
					}

				break;
			}
		})
	}

	// R3 SPECIFIC FUNCTIONS

	// FOR ADDING CUSTOM SCRIPTS

	function randomize_id( field ){
		field.value = Math.random().toString(36).replace(/[^a-z1-9]+/g, '').substr(0, 5); // 5 random  characters
	}

	function randomize_ids_for_custom_scripts(){
		let script_id_fields = FP.findAll('.fupi_r3_scr .fupi_field_id_wrap input');
		script_id_fields.forEach( field => { if ( ! field.value ) randomize_id( field ); });
	}

	// init
	if ( FP.findFirst('.fupi_r3_repeater') ) {
		randomize_ids_for_custom_scripts();
		enable_all_select2s();
		enable_section_buttons();
		enable_focusout_checks();
	}

})();

;(()=>{

	// format: KEY
	// document.addEventListener('focusout', e=>{
	// 	if ( e.target.dataset.dataformat && e.target.dataset.dataformat == 'key' ){
	// 		console.log('focusout !');
	// 	}
	// })

	document.addEventListener('keyup', e=>{
		if ( e.target.dataset.dataformat && e.target.dataset.dataformat == 'key' ){
			let reg = /^\d/, // digit at the begining
				reg2 = /[^\w]/gi, // not a special char or underscore
				txt = e.target.value;

			txt = txt.replace(reg,'');
			txt = txt.replace(reg2,'');

			e.target.value = txt;
		}
	})
})();


(()=>{

	// VIDEO POPUP

	function create_popup(){

		if ( !FP.findID('fupi_popup') ){
			FP.findID('fupi_content').insertAdjacentHTML('afterend', '<div id="fupi_popup"><div id="fupi_popup_overlay"></div><div class="fupi_inner"><button type="button" id="fupi_close_popup_btn">&times;</button><div class="fupi_content"></div></div></div>');
		}
	}

	function add_content_to_popup( btn, is_video ){

		// if the user clicked a video link add to the popup only the video
		if ( is_video ){
			let current_vid = FP.findFirst('#fupi_popup .fupi_content iframe'),
				vid_src = 'https://www.youtube-nocookie.com/embed/' + btn.dataset.srcid;

			if ( !current_vid || current_vid.src != vid_src ) {
				FP.findFirst('#fupi_popup .fupi_content').innerHTML = '<div class="fupi_video"><iframe width="560" height="315" src="' + vid_src + '" title="YouTube video player" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe><div>'
			}

		// if the user clicked a popup btn add to the popup all required data (and init a video if there is one)
		} else {
			let popup_content = btn.nextElementSibling;
			if ( popup_content.classList.contains( 'fupi_popup_content' ) ) {
				FP.findFirst('#fupi_popup .fupi_content').innerHTML = popup_content.innerHTML;
			}
		}

	}

	function show_popup(){
		FP.findID('fupi_popup').classList.remove('fupi_hidden');
	}

	function hide_popup(){
		FP.findID('fupi_popup').classList.add('fupi_hidden');
	}

	function add_once_close_listeners(){
		if ( ! FP.findID('fupi_popup').classList.contains('fupi_close_listeners_added') ) {
			FP.findID('fupi_popup').classList.add('fupi_close_listeners_added');
			FP.findID('fupi_close_popup_btn').addEventListener( 'click', e => { hide_popup() });
		}
	}

	FP.findAll('a.fupi_vid').forEach( btn => {
		btn.addEventListener( 'click', e => {
			FP.updateViewportDimensions();
			if ( FP.v.viewport.width > 600 && FP.v.viewport.height > 450 )  {
				e.preventDefault();
				create_popup();
				add_content_to_popup( btn, true );
				show_popup();
				add_once_close_listeners();
			}
		});
	})

	FP.findAll('.fupi_open_popup').forEach( btn => {
		btn.addEventListener( 'click', e => {
			create_popup();
			add_content_to_popup( btn, false );
			show_popup();
			add_once_close_listeners();
		})
	});

})();
/*
(()=>{

	// SHOW "FIRST STEPS" VIDEO AFTER PLUGIN ACTIVATION

	if ( FP.getUrlParamByName('intro') == 'true' && ! FP.findID('setting-error-settings_updated') ){
		let intro_link = FP.findID('fupi_intro_vid_link')
		if ( intro_link ) intro_link.click();
	}
})();
*/

(()=>{

	// INCREASE / DECREASE VALUE IN THE "NUMBER" FIELD AFTER CLICKS IN "PLUS" AND "MINUS" BUTTONS

	function changeNumber( action, btn ){

		let num_field = FP.findFirst( 'input[type="number"]', btn.parentElement.parentElement ),
			old_val = parseInt( num_field.value || 0 );

		if ( action == 'plus' ){
			num_field.value = old_val + ( num_field.step || 1 );
		} else {
			num_field.value = old_val - ( num_field.step || 1 );
		}
	}

	document.addEventListener('click', e=>{
		let btn = e.target;
		if ( btn.matches('tr:not(.fupi_disabled) .fupi_number_increase .dashicons') ) {
			changeNumber( 'plus', btn );
		} else if ( e.target.matches('tr:not(.fupi_disabled) .fupi_number_decrease .dashicons') ) {
			changeNumber( 'minus', btn );
		};
	})
})();

(()=>{

	// MARK TABLE ROWS WITH FIELDS ADDED BY EXTENSIONS

	let ext_fields = FP.findAll('.fupi_ext_field');
	ext_fields.forEach( field => field.closest('tr').classList.add('fupi_ext_options') );

})();

(()=>{

	// TAB NAVIGATION

	document.addEventListener('click', e=>{
		if ( e.target.classList.contains('fupi_tab') && ! e.target.classList.contains('fupi_active') ){
			FP.findFirst( '.fupi_tab.fupi_active', e.target.parentElement ).classList.remove('fupi_active');
			e.target.classList.add('fupi_active');
		}
	})

})();

(()=>{

	// ADD "EXTERNAL" DASHICON TO LINKS

	window.addEventListener( 'DOMContentLoaded', ()=>{

		let links = FP.findAll('#fupi_main_col a');

		links.forEach( link => {
			if ( ! FP.isIn( document.location.host, link.href ) && ! link.classList.contains('fupi_vid') ) {
				link.insertAdjacentHTML('beforeend', ' <span class="dashicons dashicons-external"></span>');
			}
		})

	})
})();

