(($) => {

	// HIGHLIGHT NEW MENU ITEMS

	function get_ids_from_inputs( enabled_modules_inputs ){
		return enabled_modules_inputs.map( input => {
			let id = input.id,
				mod = id.replace('fupi_tools[','');

			return mod.replace(']','');
		});
	}

	function get_curr_modules(){

		let enabled_modules_inputs = FP.findAll('.fupi_switch input:checked'),
			enabled_modules = enabled_modules_inputs ? get_ids_from_inputs( enabled_modules_inputs ) : [];

		if ( FP.findFirst('#adminmenu a[href="admin.php?page=full_picture_main"]') ) enabled_modules.push('main');

		return enabled_modules;
	}

	function get_prev_modules(){
		return JSON.parse( window.sessionStorage.getItem('fupi_enabled_modules') ) || [];
	}

	function get_new_modules( curr, prev ){
		if ( curr.length > 0 ){
			let prev_highlight_modules = get_prev_highlighted_modules(),
				curr_new = curr.filter( mod => prev.indexOf( mod ) == -1 ), // here we have only the newly added modules that need to be highlighted
				curr_new_and_old = prev_highlight_modules.concat(curr_new.filter((item) => prev_highlight_modules.indexOf(item) < 0)); // here we have the newly added AND the old highlighted (weren't clicked yet). This code is for deduplication of values

			return curr_new_and_old;
		}
		return [];
	}

	function get_prev_highlighted_modules(){
		let cookie_val = FP.readCookie('fupi_new_adm_pages');
		return cookie_val ? JSON.parse(cookie_val) : [];
	}

	function check_for_new_menu_items(){

		let curr_modules = get_curr_modules(),
			prev_modules = get_prev_modules(),
			new_modules = get_new_modules(curr_modules, prev_modules);

		// save NEW & CURRENT modules
		window.sessionStorage.setItem('fupi_enabled_modules', JSON.stringify( curr_modules )); // curr are saved in sessionStorage since they are needed only when the "tools" page is refreshed
		FP.setCookie('fupi_new_adm_pages', JSON.stringify( new_modules ), 365); // new ones are saved in cookie to be checked on each fupi settings page
	}

	function highlight_new_menu_items(){

		let curr_fupi_page = FP.getUrlParamByName('page');

		var modules = get_prev_highlighted_modules();

		modules.forEach( mod => {

			// if currently viewed page is a newly added page
			if ( curr_fupi_page == 'full_picture_' + mod ) {
				// only remove it from highlighted items
				var new_modules = modules.filter( m => m != mod );
				FP.setCookie('fupi_new_adm_pages', JSON.stringify( new_modules ), 365);
			// if currently viewed page is NOT a newly added page
			} else {
				// highlight side menu item
				let side_menu_item = FP.findFirst('#fupi_page_nav a[href*="full_picture_' + mod + '"]');
				if ( side_menu_item ) side_menu_item.classList.add('fupi_new_menu_item');
			}
		} );
	}

	function remove_highlight(){
		FP.deleteCookie('fupi_new_adm_pages');
		let menu_items = FP.findAll('.fupi_new_menu_item');
		menu_items.forEach( item => item.classList.remove('fupi_new_menu_item') );
	}

	function showNotification(){
		let new_menu_items = FP.findAll('.fupi_new_menu_item'),
			fupi_main = FP.findID('fupi_main');

		if ( new_menu_items.length > 0 ) {
			if ( fupi_main ) fupi_main.insertAdjacentHTML( 'afterend', '<div id="fupi_new_pages_in_menu" class="notice notice-warning"><p><strong>New settings pages are now added to the WordPress main menu in the Full Picture section.</strong> <button type="button" class="button button-secondary" id="fupi_remove_highlight_btn">Remove highlight</button></p></div>');

			$('#fupi_remove_highlight_btn').click( ()=>{
				$('#fupi_new_pages_in_menu').slideUp('fast');
				remove_highlight();
			} );
		}
	}

	// ON PAGELOAD...

	// only on "tools" page
	if ( FP.getUrlParamByName('page') == 'full_picture_tools' ) {
		// save list of currently enabled modules if we don't have it yet
		if ( ! window.sessionStorage.getItem('fupi_enabled_modules' ) ) window.sessionStorage.setItem('fupi_enabled_modules', JSON.stringify( get_curr_modules() ));
		// check for new menu items if there is a "settings updated" notice
		if ( FP.findID('setting-error-settings_updated') ) check_for_new_menu_items();
	}

	// highlight new items and show notice
	highlight_new_menu_items();
	// showNotification();

})( jQuery );
