// SCRIPT REPEATER

(() => {

	function remove_or_clear_section(target){
		let closest_section_wrap = target.closest('.fupi_r3_section'),
			siblingSections = FP.getSiblings( closest_section_wrap );

		siblingSections = siblingSections.filter(sect => sect.classList.contains('fupi_r3_section')); // get only r3 sections

		// check if this is the only section on this level
		if ( siblingSections.length > 0 ) {
			FP.remove(closest_section_wrap);
		} else {
			clear_section(closest_section_wrap);
		}
	}

	function add_section(target){

		let current_section = target.closest('.fupi_r3_section'),
			cloned_section = current_section.cloneNode(true),
			scr_id_field = FP.findFirst('.fupi_r3_scr .fupi_field_id_wrap input', cloned_section);

		cloned_section = clear_section(cloned_section);
		cloned_section = remove_extra_repeaters(cloned_section);
		current_section.parentElement.insertBefore(cloned_section, current_section.nextSibling);

		if ( scr_id_field ) randomize_id( scr_id_field );
	}

	function destroy_all_select2s(){
		(($)=>{
			if ( $.isFunction($.fn.select2) ){
				$('.fupi_field_type_select2').each( (i, wrap) => {
					$wrap = $(wrap);
					if ( $wrap.hasClass('fupi_select2_enabled') ) {
						$wrap.find('select').select2('destroy');
						$wrap.removeClass('fupi_select2_enabled');
					}
				});
			}
		})(jQuery)
	}

	function enable_all_select2s(){
		jQuery(document).ready(function($) {
			if ( $.isFunction($.fn.select2) ){
				$('.fupi_field_type_select2').each( (i, wrap) => {
					$wrap = $(wrap);
					$wrap.find('select').select2();
					$wrap.addClass('fupi_select2_enabled');
				})
			}
		});
	}

	function rename_fields_in_section( sections_wrap, parent_sect_nums ){

		if ( ! parent_sect_nums ) parent_sect_nums = [];

		// console.log('sections_wrap', sections_wrap);

		// get all the sections that are direct descendants of the section wrap
		let sibling_sections = FP.nl2Arr( sections_wrap.children ),
			sections = sibling_sections ? sibling_sections.filter( siblig => siblig.classList.contains('fupi_r3_section') ) : [];

		// console.log('same level sections', sections);

		sections.forEach( (section, sect_i) => {

			let current_sect_nums = [...parent_sect_nums, sect_i];

			// get all the field wrappers tha are direct descendants of the section
			let section_children = FP.nl2Arr( section.children ),
				field_wrappers = section_children ? section_children.filter( child => child.classList.contains('fupi_r3_field') && ! child.classList.contains('fupi_field_type_label') ) : [];

			field_wrappers.forEach( ( field_wrap, field_i ) => {

				// if field wrapper contains a repeater run renaming function on its content
				if ( field_wrap.classList.contains('fupi_field_type_r3') ){

					rename_fields_in_section(field_wrap, current_sect_nums );

				// if the field wrapper contains form fields then modify their names and ids
				} else {

					let field = FP.findFirst('input, textarea, select', field_wrap),
						name_parts = field.name.split(/\]\[\d*?\]\[/g) || []; // this gives us strings with square brackets, like '][3]['. We don't need anything more than that. We use it only to count how many occurances of digits appear

					if (name_parts.length - 1 <= current_sect_nums.length ){ // there should always be 1 more name part than numbers to fill in

						// here we create a name and id
						let new_name = name_parts.shift(); // we take the first name part (this also lowers the number of elements in the array)
						name_parts.forEach( ( part, i ) => { new_name += '][' + current_sect_nums[i] + '][' + part; } );

						// console.log('new field name for ' + field.name + ' is ' + new_name);

						field.name = new_name;
						field.id = new_name;
					} else {
						console.error('name can\'t be constructed for ', field, ' with: ', name_parts, current_sect_nums );
					}
				};
			} );
		} );
	}

	function remove_extra_repeaters(section){
		let extra_section = FP.findFirst('.fupi_r3_repeater + .fupi_r3_repeater', section);
		if ( extra_section ) {
			FP.remove(extra_section);
			return remove_extra_repeaters(section);
		} else {
			return section;
		}
	}

	function clear_section(section){

		FP.findAll('textarea', section).forEach( area => area.value = '' );
		FP.findAll('input[type="checkbox"]', section).forEach( box => box.checked = false );
		FP.findAll('input[type="text"]', section).forEach( field => field.value = '' );
		FP.findAll('input[type="number"]', section).forEach( field => field.value = 0 );
		FP.findAll('input[type="password"]', section).forEach( field => field.value = '' );
		FP.findAll('select', section).forEach( sel => sel.value = FP.findFirst('option', sel).value );

		FP.findAll('.field_empty', section).forEach( field => field.classList.remove('field_empty') );

		return section;
	}

	function enable_section_buttons() {

		FP.findID('fupi_settings_form').addEventListener('click', e => {

			let btn = e.target;

			if ( btn.classList.contains('fupi_r3_btn') ){

				destroy_all_select2s();
				let section_wrap = btn.closest('td');

				if ( btn.classList.contains('fupi_btn_remove') ){
					remove_or_clear_section(btn);
					rename_fields_in_section(section_wrap);
				} else {
					add_section(btn);
					rename_fields_in_section(section_wrap);
				}

				enable_all_select2s();
			}
		})
	}

	function get_attributes_from_str( str ){

		// return an object with key-value pairs
		//
		// string example: 'data-param='abc def ghi' async src="https://www.googletagmanager.com/gtag/js?id=UA-11111111-1" data-param2="abc" id="anID" class="class1 class2"'


		let parts = str.trim().split(' '), // we break the string into parts by 'space' (spaces can occur in unexpected places! see example above)
			ok_parts = parts.filter( v => v ), // and remove empty values from array
			ret_val = {};

		for ( var i = 0; i < ok_parts.length; i++ ) {

			let part = ok_parts[i],
				eq_index = part.indexOf('='),
				last_char = part[part.length-1];

			// if we have an equal sign
			// (this means that the value before the equal sign is the key and the one after is the value)
			if ( eq_index != -1 ){

				// we check if the last element of the current part is an apostrophe. If it isn\'t then it means we split the value in two or more pieces and need to join it with the next string parts
				while ( ! ( last_char == '"' || last_char == "'" ) && i+1 <= ok_parts.length ) {
					part += ' ' + ok_parts[i+1];
					last_char = part[part.length-1];
					i++;
				}

				// if we have the key and the whole value
				if ( last_char == '"' || last_char == "'" ) {
					// we save the key and value pair
					let key_val = part.split(/=(.*)/s); // split on the first occurance of "=" sign. They may sometimes show up in values too
					ret_val[key_val[0]]= key_val[1];
				}

			// if we don't have an equal sign this is a single value
			} else {
				ret_val[part] = true;
			}
		}

		console.log('ret_val', ret_val);

		return ret_val;

	}

	function reformat_custom_scripts( script ){

		// remove all ending "</script>" tags
		script = script.replaceAll('</script>', '');

		// find all <script[something or nothing]> tags
		let start_regex = /<script[\s\S]*?>/gi,
			matches = script.match(start_regex),
			replacements = [],
			has_error = false;

		// get their attributes
		if ( Array.isArray( matches ) && matches.length > 0 ) matches.forEach( txt => {

			// remove "<script" and final ">"
			txt = txt.replace('<script','');
			txt = txt.slice(0, -1);

			if ( txt.length == 0 ) {

				replacements.push('');

			} else {

				// transform the "<script [attributes]>" into "getScript()"
				let attributes_obj = get_attributes_from_str(txt),
					src = attributes_obj.src ? attributes_obj.src : false,
					attrs_str = '';

				if ( ! src ) {
					console.error('Cannot re-format the <script> elements in the code. Missing "src" attribute.');
					has_error = true;
				}

				for (const [key, val] of Object.entries(attributes_obj)) {
					if ( key != 'src' ) attrs_str += '"' + key + '": ' + val + ', ';
				}

				replacements.push([src, attrs_str]);
			}
		} );

		if ( has_error ) return false;

		// prepare elements to replace <string [something or nothing]> in the original script
		replacements.forEach( ( repl, i ) =>{
			if ( repl ) replacements[i] = 'FP.getScript(' + repl[0] + ', false, {' + repl[1] + '});'; // repl[0] & repl[1] are already properly formatted
		} )

		// replace elements in string one by one

		let i = 0;

		return script.replace(start_regex, ()=>{ return replacements[i++]; });

		// e.target.value = e.target.value.replaceAll(/<.*?script[\s\S]*?>/g, '');
	}

	function enable_focusout_checks(){
		document.addEventListener( 'focusout', e => {

			switch (e.target.tagName) {
				case 'INPUT':
				case 'TEXTAREA':
				case 'SELECT':

					let r3_field_wrap = e.target.closest('.fupi_r3_field');

					if ( r3_field_wrap ) {

						// mark as empty
						e.target.value ? r3_field_wrap.classList.remove('field_empty') : r3_field_wrap.classList.add('field_empty');

						// sanitize script
						if ( e.target.tagName == 'TEXTAREA' ) {

							let new_value = reformat_custom_scripts( e.target.value ); // e.target.value = e.target.value.replaceAll(/<.*?script[\s\S]*?>/g, '');
							if ( new_value ) e.target.value = new_value;
						}

						// sanitize script id
						if ( FP.isIn( '[id]', e.target.name ) ) e.target.value = e.target.value.replace(/[^a-z0-9_]/g, "");
					}

				break;
			}
		})
	}

	// R3 SPECIFIC FUNCTIONS

	// FOR ADDING CUSTOM SCRIPTS

	function randomize_id( field ){
		field.value = Math.random().toString(36).replace(/[^a-z1-9]+/g, '').substr(0, 5); // 5 random  characters
	}

	function randomize_ids_for_custom_scripts(){
		let script_id_fields = FP.findAll('.fupi_r3_scr .fupi_field_id_wrap input');
		script_id_fields.forEach( field => { if ( ! field.value ) randomize_id( field ); });
	}

	// init
	if ( FP.findFirst('.fupi_r3_repeater') ) {
		randomize_ids_for_custom_scripts();
		enable_all_select2s();
		enable_section_buttons();
		enable_focusout_checks();
	}

})();
