// REPEATER

(() => {

	let textareas = FP.findAll('.fupi_repeater'),
		template = '<div class="fupi_repeater_line_wrap fupi_repeater_template">'+
		 	'<div class="fupi_repeater_fields_wrap fupi_ok">' +
				'<div class="fupi_repeater_p1_wrap">' +
					'<input class="fupi_repeater_field fupi_repeater_p1 fupi_required" type="text">' +
					'<span class="fupi_error_msg fupi_error_msg_p1" style="display: none;"></span>' +
					'<span class="fupi_required_msg fupi_required_msg_p1" style="display: none;"></span>' +
				'</div>' +
				'<div class="fupi_repeater_p2_wrap">' +
					'<input class="fupi_repeater_field fupi_repeater_p2" type="text">' +
					'<span class="fupi_error_msg fupi_error_msg_p2" style="display: none;"></span>' +
					'<span class="fupi_required_msg fupi_required_msg_p2" style="display: none;"></span>' +
				'</div>' +
			'</div>'+
			'<div class="fupi_repeater_btns_wrap">' +
				'<button type="button" class="fupi_repeater_btn fupi_btn_remove dashicons dashicons-minus"></button>' +
				'<button type="button" class="fupi_repeater_btn fupi_btn_add dashicons dashicons-plus-alt2"></button>' +
			'</div>' +
		'</div>';

	function add_repeater_fields( textarea, placement, line_content, is_disabled ) {

		// Add template to page
		placement.insertAdjacentHTML('afterend', template);

		let content = line_content ? line_content.split('@') : ['',''],
			addedTemplate = FP.findFirst('.fupi_repeater_template'),
			field1 = FP.findFirst('.fupi_repeater_p1', addedTemplate),
			field2 = FP.findFirst('.fupi_repeater_p2', addedTemplate),
			field2_req_text = textarea.dataset.field2_req;

		// Fill it with data

		// field 1
		field1.value = content[0];
		field1.disabled = is_disabled;
		field1.placeholder = textarea.dataset.field1_txt;
		field1.dataset.type = textarea.dataset.field1_type;
		FP.findFirst( '.fupi_error_msg_p1', addedTemplate ).textContent = textarea.dataset.field1_err;
		FP.findFirst( '.fupi_required_msg_p1', addedTemplate ).textContent = textarea.dataset.field1_req;

		// field 2
		field2.value = content[1] || '';
		field2.disabled = is_disabled;
		field2.placeholder = textarea.dataset.field2_txt;
		FP.findFirst( '.fupi_error_msg_p2', addedTemplate ).textContent = textarea.dataset.field2_err || '';
		if ( field2_req_text ) {
			field2.classList.add('fupi_required');
			FP.findFirst( '.fupi_required_msg_p2', addedTemplate ).textContent = field2_req_text;
		}

		// Remove template class
		addedTemplate.classList.remove('fupi_repeater_template');
	}

	function enable_line_buttons() {

		let form = FP.findID('fupi_settings_form');

		if ( form ) {
			form.addEventListener('click', e => {
				let target = e.target;
				if ( target.classList.contains('fupi_repeater_btn') ) {

					let btns_wrap = target.closest( '.fupi_repeater_line_wrap'),
						textarea = btns_wrap.previousElementSibling;

					if ( textarea && ! textarea.disabled ) {

						// remove line
						if ( target.classList.contains('fupi_btn_remove') ) {

							let fields_line = target.closest('.fupi_repeater_line_wrap'),
							repeater_section = target.closest('.fupi_repeater_section');

							FP.remove(fields_line);
							update_textarea(false, repeater_section);

						// add line
						} else {
							let fupi_repeater_section = target.closest('.fupi_repeater_section'),
							textarea = FP.findFirst('textarea', fupi_repeater_section),
							line_with_clicked_btn = target.closest('.fupi_repeater_line_wrap');

							add_repeater_fields(textarea, line_with_clicked_btn, false, false)
						}
					}
				}
			})
		}
	}

	function create_repeater_fields () {

		textareas.forEach( textarea => {

			let value = textarea.value,
				lines = value ? value.split(/\r?\n/) : [''], // split lines into array (if there are no lines create an empty array)
				lines_reversed = lines.reverse(),
				is_disabled = textarea.disabled;

			// create fields for each line
			lines_reversed.forEach( line => add_repeater_fields( textarea, textarea, line, is_disabled ) )

		} )
	}

	function is_selector_valid ( sel ) {
		try { document.querySelector( sel ) } catch (e) { return false; }
		return true;
	}

	function toggle_error_msg( field ) {

		let is_field_1 = field.classList.contains('fupi_repeater_p1'),
			error_msg = FP.findFirst( '.fupi_error_msg', field.parentElement );

		// for field 1
		if ( is_field_1 ) {

			// if field value is a CSS selector
			if ( field.dataset.type == 'selector' ) {

				if ( is_selector_valid( field.value ) ) {
					error_msg.style.display = 'none';
					field.classList.remove('fupi_error');
				} else {
					field.classList.add('fupi_error');
					error_msg.style.display = 'block';
				}
			}

		// if field 2
		} else {

			// if string contains characters from outside this list
			if ( /[^A-Za-z0-9 \-\_\[\]]/.test( field.value ) ) {
				field.classList.add('fupi_error');
				error_msg.style.display = 'block';
			} else {
				error_msg.style.display = 'none';
				field.classList.remove('fupi_error');
			}
		}
	}

	function toggle_required_msg( field ) {

		if ( field.classList.contains('fupi_required') ) {

			let req_msg = FP.findFirst( '.fupi_required_msg', field.parentElement );

			if ( field.value ) {
				field.classList.remove('fupi_empty');
				req_msg.style.display = 'none';
			} else {
				field.classList.add('fupi_empty');
				req_msg.style.display = 'block';
			}
		}
	}

	function mark_ok_lines( field ) {

		// check if all the required fields in this "line" are filled in correctly

		let fields_wrap = field.closest('.fupi_repeater_fields_wrap'),
		 	repeater_fields_with_errors = FP.findAll('.fupi_repeater_field.fupi_empty, .fupi_repeater_field.fupi_error', fields_wrap);

		repeater_fields_with_errors.length == 0 ? fields_wrap.classList.add('fupi_ok') : fields_wrap.classList.remove('fupi_ok');
	}

	function update_textarea( field, repeater_section = false) {

		// copy to textarea values of all correctly filled in fields

		let r_section = repeater_section || field.closest('.fupi_repeater_section'),
			field_wraps = FP.findAll('.fupi_ok', r_section), // get all fields from lines that have no erros
			textarea = FP.findFirst('textarea', r_section),
			new_val = '';

		if ( field_wraps.length > 0 ) {
			field_wraps.forEach( wrap => {

				let fields = FP.findAll('input', wrap).filter( input => !!input.value );

				if ( fields.length > 0 ) {
					if ( fields.length > 1 ) {
						new_val += fields[0].value.trim() + '@' + fields[1].value.trim()  + "\n";
					} else {
						new_val += fields[0].value.trim() + "\n";
					}
				}

			} )
		}

		textarea.value = new_val.trim();

		// trigger keyup event on textarea (for other functions to work properly)
		textarea.click();
	}

	// init
	if ( textareas ) {

		create_repeater_fields();
		enable_line_buttons();

		// when repeater fields lose focus...

		document.addEventListener('focusout', e => {

			let field = e.target;

			if ( field.classList && field.classList.contains('fupi_repeater_field') ){
				toggle_error_msg( field );
				toggle_required_msg( field );
				mark_ok_lines( field );
				update_textarea( field );
			}
		})
	}
})();
