//

(() => {

	let fupi_settings_form 	= FP.findID('fupi_settings_form'),
		fupi_admin_page 	= FP.getUrlParamByName('page'),
		wp_menu_items 		= FP.findFirst('#toplevel_page_full_picture_tools ul'),
		content_el 			= FP.findID('fupi_content');

	function mark_tables_with_classes() {

		let tables = FP.findAll('#fupi_settings_form>table.form-table');

		tables.forEach( ( table, i ) =>{
			let table_class = 'fupi_tools_list';
			switch (i) {
				case 0:
					table.classList.add( 'fupi_table_grid' );
				break;
				case 1:
					table_class = 'fupi_privacy_list';
				break;
				case 2:
					table_class = 'fupi_ext_list';
				break;
			}
			table.classList.add( table_class );
		})
	}

	function tools_enabled(){
		return FP.findAll('.fupi_tools_list tr:not(.fupi_disabled) input[type="checkbox"]:checked').length > 0;
	}

	function show_hide_next_btn_on_tools_toggle(){
		document.addEventListener('change', e=>{
			show_or_hide_next_btn();
		});
	}

	function show_or_hide_next_btn(){
		let checkboxes = FP.findAll('.fupi_tools_list tr:not(.fupi_disabled) input[type="checkbox"]');

		if ( checkboxes.some(chk => chk.checked) ){
			content_el.classList.add('fupi_has_toggled_tools');
		} else {
			content_el.classList.remove('fupi_has_toggled_tools');
		}
	}

	function listen_to_clicks_on_buttons(){

		// CANCEL
		FP.findID('fupi_cancel_btn').addEventListener('click', ()=>{ change_step('list'); })

		// ADD TOOLS
		FP.findID('fupi_add_tools_btn').addEventListener('click', ()=>{ change_step('first'); })

		// NEXT
		FP.findID('fupi_next_btn').addEventListener('click', ()=>{ change_step('next'); });

		// PREV
		FP.findID('fupi_prev_btn').addEventListener('click', ()=>{ change_step('prev'); });

		document.body.addEventListener('click', e=>{
			let target = e.target;

			if ( target.parentElement.classList.contains('fupi_change_modules_link') ) target = target.parentElement;

			if ( target.classList.contains('fupi_change_modules_link') ) {
				switch (target.id) {
					case 'fupi_change_tools':
						show_step(11);
					break;
					case 'fupi_change_privacy':
						show_step(22);
					break;
					case 'fupi_change_ext':
						show_step(33);
					break;
				}
			}
		})
	}

	function change_step( step ){

		// get current step number
		let curr_step = parseInt( content_el.dataset.step ),
			next_step = curr_step;

		// get what step to show
		if ( step == 'list' ) next_step = 0; // screen with enabled modules
		if ( step == 'first' ) next_step = 1;
		if ( step == 'next' ) ++next_step;
		if ( step == 'prev' ) --next_step;

		show_step(next_step);
	}

	function show_step(step){
		content_el.dataset.step = step;
		show_or_hide_next_btn();
		window.scrollTo({top: 0, behavior: 'smooth'});
	}

	function generate_list_of_enabled_modules(){

		let has_ext = false,
			has_privacy = false,
			has_tools = false;

		['tools', 'privacy', 'ext'].forEach( module_type => {


			let enabled_modules_wrap = FP.findFirst('#fupi_' + module_type + '_faux tbody'),
				enabled_modules = FP.findAll('.fupi_' + module_type + '_list tr:not(.fupi_disabled) input[type="checkbox"]:checked');

			enabled_modules.forEach( module => {

				let tr_el = module.closest('tr').cloneNode(true)
				 	label = FP.findFirst('label', tr_el),
					module_name = FP.findFirst('input', label).name,
					module_id = module_name.replace('fupi_tools[',''),
					after_field_info = FP.findFirst('.fupi_after_field', tr_el);

				module_id = module_id.replace(']', '');

				let menu_link = FP.findFirst( 'a[href*=full_picture_' + module_id + ']', wp_menu_items);

				if ( menu_link ){

					let new_link = document.createElement('a');

					new_link.href = menu_link.href;
					new_link.classList.add('button', 'button-secondary'),
					new_link.textContent = 'Settings';

					label.parentElement.insertBefore(new_link, label)
				} else {

					let no_link_info = document.createElement('span');

					no_link_info.textContent = 'This module has no settings page'
					no_link_info.classList.add('fupi_no_link_info');

					label.parentElement.insertBefore(no_link_info, label);
				}

				if ( label ) FP.remove( label );
				if ( after_field_info ) FP.remove( after_field_info );

				enabled_modules_wrap.appendChild(tr_el);

				if ( module_type == 'tools' ) has_tools = true;
				if ( module_type == 'privacy' ) has_privacy = true;
				if ( module_type == 'ext' ) has_ext = true;
			});
		} );

		// if there are any enabled ext or privacy modules hide the "no modules enabled" notices
		if ( has_privacy ) FP.findID('fupi_no_privacy_modules_selected').style.display = 'none';
		if ( has_ext ) FP.findID('fupi_no_modules_selected').style.display = 'none';

		// and finally show the whole generated list
		content_el.classList.add('fupi_has_enabled_tools');
	}


	if ( fupi_admin_page && fupi_admin_page == 'full_picture_tools' && fupi_settings_form) {

		mark_tables_with_classes();
		listen_to_clicks_on_buttons();

		if ( ! tools_enabled() ) {
			change_step('first');
		} else {
			generate_list_of_enabled_modules();
		}

		show_hide_next_btn_on_tools_toggle();
	};

})();
