<?php

/**
 * Plugin Name:       Full Picture
 * Plugin URI:        https://fullpictureplugin.com/
 * Description:       Privacy-Focused Tag Manager & Cookie Notice. Add any custom JS or use 17 ready integrations with analytics and marketing tools.
 * Version:           4.1.0
 * Requires at least: 5.0
 * Requires PHP:      7.2
 * Author:            Krzysztof Planeta
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       fupi
 * Domain Path:       /languages
*/
// If this file is called directly, abort.
if ( !defined( 'ABSPATH' ) ) {
    exit;
}

if ( function_exists( 'fupi_fs' ) ) {
    fupi_fs()->set_basename( false, __FILE__ );
} else {
    // DO NOT REMOVE THIS IF, IT IS ESSENTIAL FOR THE `function_exists` CALL ABOVE TO PROPERLY WORK.
    
    if ( !function_exists( 'fupi_fs' ) ) {
        // this will be removed in FREE version
        // Create a helper function for easy SDK access.
        function fupi_fs()
        {
            global  $fupi_fs ;
            
            if ( !isset( $fupi_fs ) ) {
                // Include Freemius SDK.
                require_once dirname( __FILE__ ) . '/freemius/start.php';
                $fupi_fs = fs_dynamic_init( array(
                    'id'             => '5405',
                    'slug'           => 'full-picture-analytics-cookie-notice',
                    'premium_slug'   => 'full-picture-premium',
                    'type'           => 'plugin',
                    'public_key'     => 'pk_2aee883bf3a3ae5559a119e92c744',
                    'is_premium'     => false,
                    'premium_suffix' => 'Premium',
                    'has_addons'     => false,
                    'has_paid_plans' => true,
                    'anonymous_mode' => !function_exists( 'fupi_premium_test__premium_only' ),
                    'trial'          => array(
                    'days'               => 7,
                    'is_require_payment' => true,
                ),
                    'menu'           => array(
                    'slug'        => 'full_picture_tools',
                    'first-path'  => 'admin.php?page=full_picture_tools',
                    'contact'     => false,
                    'support'     => false,
                    'affiliation' => false,
                    'pricing'     => false,
                ),
                    'is_live'        => true,
                ) );
            }
            
            return $fupi_fs;
        }
        
        // Init Freemius.
        fupi_fs();
        // Signal that SDK was initiated.
        do_action( 'fupi_fs_loaded' );
    }
    
    define( 'FUPI_VERSION', '4.1.0' );
    define( 'FUPI_URL', plugin_dir_url( __FILE__ ) );
    define( 'FUPI_PATH', __DIR__ );
    // REGISTER MODULES
    $is_lite = false;
    $is_pro = false;
    // order is important - modules will show up in this order in the side menu (except the 1st one which is hardcoded to always go 1st)!
    define( 'FUPI_MODULES', [
        [
        'tools',
        // slug
        'Modules',
        // title - Use Camel Case For Consistency !
        true,
        // can be used in current plan
        false,
        // is premium code
        0,
        // is standard module (0), is cookie notice (1) or is a module without settings page (2)
        false,
    ],
        [
        'ga41',
        'Google Analytics 4',
        true,
        false,
        0,
        false
    ],
        [
        'ga42',
        'Google Analytics 4 #2',
        $is_lite,
        true,
        0,
        false
    ],
        [
        'ga1',
        'Google Analytics UA',
        true,
        false,
        0,
        false
    ],
        [
        'ga2',
        'Google Analytics UA #2',
        $is_lite,
        true,
        0,
        false
    ],
        [
        'gads',
        'Google Ads',
        true,
        false,
        0,
        false
    ],
        [
        'gopt',
        'Google Optimize',
        true,
        false,
        0,
        false
    ],
        [
        'fbp1',
        'Facebook Pixel',
        true,
        false,
        0,
        false
    ],
        [
        'fbp2',
        'Facebook Pixel #2',
        $is_lite,
        true,
        0,
        false
    ],
        [
        'mads',
        'Microsoft Advertising',
        true,
        false,
        0,
        false
    ],
        [
        'clar',
        'Microsoft Clarity',
        true,
        false,
        0,
        false
    ],
        [
        'linkd',
        'LinkedIn Insight',
        true,
        false,
        0,
        false
    ],
        [
        'pin',
        'Pinterest Ads',
        true,
        false,
        0,
        false
    ],
        [
        'tik',
        'TikTok Pixel',
        true,
        false,
        0,
        false
    ],
        [
        'twit',
        'Twitter Ads',
        true,
        false,
        0,
        false
    ],
        [
        'hotj',
        'Hotjar',
        true,
        false,
        0,
        false
    ],
        [
        'insp',
        'Inspectlet',
        true,
        false,
        0,
        false
    ],
        [
        'pla',
        'Plausible',
        true,
        false,
        0,
        false
    ],
        [
        'cegg',
        'Crazy Egg',
        true,
        false,
        0,
        false
    ],
        [
        'sbee',
        'Splitbee',
        true,
        false,
        0,
        false
    ],
        [
        'gtm',
        'Google Tag Manager',
        true,
        false,
        0,
        false
    ],
        [
        'cscr',
        'Custom Scripts ',
        true,
        false,
        0,
        false
    ],
        [
        'main',
        'General Settings',
        true,
        false,
        0,
        false
    ],
        [
        'woo',
        'WooCommerce Tracking',
        $is_pro,
        true,
        0,
        false
    ],
        [
        'trackmeta',
        'Metadata Tracking',
        true,
        true,
        0,
        false
    ],
        [
        'cook',
        'Cookie Notice',
        true,
        false,
        1,
        false
    ],
        [
        'iframeblock',
        'Block Iframes',
        true,
        false,
        0,
        false
    ],
        [
        'privex',
        'Privacy Policy Extras',
        true,
        false,
        0,
        false
    ],
        [
        'geo',
        'Geolocation',
        true,
        false,
        0,
        false
    ],
        [
        'labelpages',
        'Page Labels',
        true,
        true,
        2,
        false
    ],
        [
        'track404',
        'Broken Links Tracking',
        true,
        false,
        0,
        false
    ]
    ] );
    // CUSTOM FP ICON
    function fupi_fs_custom_icon()
    {
        return plugin_dir_path( __FILE__ ) . 'admin/settings/img/fp_logo_2_160.png';
    }
    
    fupi_fs()->add_filter( 'plugin_icon', 'fupi_fs_custom_icon' );
    // ACTIVATE
    function activate_fupi()
    {
        require_once plugin_dir_path( __FILE__ ) . 'includes/class-fupi-activator.php';
        Fupi_Activator::activate();
    }
    
    register_activation_hook( __FILE__, 'activate_fupi' );
    // DEACTIVATE
    function deactivate_fupi()
    {
        require_once plugin_dir_path( __FILE__ ) . 'includes/class-fupi-deactivator.php';
        Fupi_Deactivator::deactivate();
    }
    
    register_deactivation_hook( __FILE__, 'deactivate_fupi' );
    // ADD SETTINGS LINK TO THE ENTRY ON PLUGINS LIST
    function fupi_plugin_settings_link( $links )
    {
        $settings_link = '<a href="admin.php?page=full_picture_tools">Settings</a>';
        array_unshift( $links, $settings_link );
        return $links;
    }
    
    add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'fupi_plugin_settings_link' );
    /**
     * The core plugin class that is used to define internationalization,
     * admin-specific hooks, and public-facing site hooks.
     */
    require plugin_dir_path( __FILE__ ) . 'includes/class-fupi.php';
    /**
     * Begins execution of the plugin.
     *
     * Since everything within the plugin is registered via hooks,
     * then kicking off the plugin from this point in the file does
     * not affect the page life cycle.
     *
     * @since    1.0.0
     */
    function run_fupi()
    {
        $plugin = new Fupi();
        $plugin->run();
    }
    
    run_fupi();
}
