<?php

class Fupi
{
    protected  $loader ;
    public function get_loader()
    {
        return $this->loader;
    }
    
    protected  $plugin_name ;
    public function get_plugin_name()
    {
        return $this->plugin_name;
    }
    
    protected  $version ;
    public function get_version()
    {
        return $this->version;
    }
    
    public function __construct()
    {
        
        if ( defined( 'FUPI_VERSION' ) ) {
            $this->version = FUPI_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        
        $this->plugin_name = 'full_picture';
        $this->tools = get_option( 'fupi_tools' );
        $this->main = get_option( 'fupi_main' );
        //delete_option('fupi_versions');
        $this->fupi_versions = get_option( 'fupi_versions' );
        $this->woo = get_option( 'fupi_woo' );
        $this->perform_updates();
        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();
    }
    
    private function perform_updates()
    {
        // this is checked every time any WP page loads (admin and front)
        // GET PREVIOUS VERSION
        $prev_version = '1.0.0';
        // default
        // probably a new install or updated from version before v4
        
        if ( empty($this->fupi_versions) ) {
            // maybe a new install or updated from 1 (it had no "tools")
            
            if ( empty($this->tools) ) {
                // is a new install
                if ( empty($this->main) ) {
                    $prev_version = $this->version;
                }
                // is updated from v2
            } else {
                $prev_version = '2.0.0';
            }
            
            // updated from after v4 or newer
        } else {
            $prev_version = $this->fupi_versions[1];
            
            if ( version_compare( $prev_version, '4.1.0' ) ) {
                $date = new DateTime();
                $install_date = $date->getTimestamp();
            } else {
                $install_date = $this->fupi_versions[0];
            }
        
        }
        
        update_option( 'fupi_versions', array( $install_date, $this->version ) );
        // DO UPDATES STEP BY STEP
        // UPDATE TO V2
        
        if ( version_compare( $prev_version, '2.0.0' ) == -1 ) {
            // we check if cookie notice is enabled
            
            if ( isset( $this->main['cookie_notice'] ) ) {
                $this->main['cook'] = '1';
                // cookies are enabled
                add_option( 'fupi_cook', $this->main );
                // cookies get separate settings page
            }
            
            add_option( 'fupi_tools', $this->main );
            // tools get separate settings page
        }
        
        // UPDATE TO V4
        
        if ( version_compare( $prev_version, '4.0.0' ) == -1 ) {
            $this->tools = get_option( 'fupi_tools' );
            // we need to take it again since it could have been updated by previous update
            // track 404
            
            if ( isset( $this->main['track_404'] ) ) {
                $this->tools['track404'] = 1;
                if ( isset( $this->main['redirect_404'] ) ) {
                    add_option( 'fupi_track404', [
                        'redirect_404' => $this->main['redirect_404'],
                    ] );
                }
            }
            
            // label pages
            if ( isset( $this->main['label_pages'] ) ) {
                $this->tools['labelpages'] = 1;
            }
            // label pages
            $this->tools['privex'] = 1;
            // track custom meta
            
            if ( isset( $this->main['custom_data_ids'] ) ) {
                $this->tools['trackmeta'] = 1;
                add_option( 'fupi_trackmeta', [
                    'custom_data_ids' => $this->main['custom_data_ids'],
                ] );
            }
            
            // geolocation & iframes blocking
            
            if ( !empty($this->tools['cook']) ) {
                $this->tools['geo'] = 1;
                $this->tools['iframeblock'] = 1;
                $cook = get_option( 'fupi_cook' );
                
                if ( !empty($cook) ) {
                    // geo options
                    $geo_opts = [];
                    if ( !empty($cook['geo']) ) {
                        $geo_opts['geo'] = $cook['geo'];
                    }
                    if ( !empty($cook['cf_worker_url']) ) {
                        $geo_opts['cf_worker_url'] = $cook['cf_worker_url'];
                    }
                    if ( !empty($cook['ipdata_api_key']) ) {
                        $geo_opts['ipdata_api_key'] = $cook['ipdata_api_key'];
                    }
                    if ( !empty($cook['skip_robots']) ) {
                        $geo_opts['skip_robots'] = $cook['skip_robots'];
                    }
                    if ( !empty($cook['remember_geo']) ) {
                        $geo_opts['remember_geo'] = $cook['remember_geo'];
                    }
                    if ( !empty($cook['save_resp']) ) {
                        $geo_opts['save_resp'] = $cook['save_resp'];
                    }
                    add_option( 'fupi_geo', $geo_opts );
                    // iframe blocking options
                    $iframe_opts = [];
                    if ( !empty($cook['iframe_img']) ) {
                        $iframe_opts['iframe_img'] = $cook['iframe_img'];
                    }
                    if ( !empty($cook['iframe_lazy']) ) {
                        $iframe_opts['iframe_lazy'] = $cook['iframe_lazy'];
                    }
                    add_option( 'fupi_iframeblock', $iframe_opts );
                }
            
            }
            
            // update in db
            update_option( 'fupi_tools', $this->tools );
        }
        
        // UPDATE TO V4.1
        
        if ( version_compare( $prev_version, '4.1.0' ) == -1 ) {
            // COOKIE NOTICE UPDATES
            $cook_opts = get_option( 'fupi_cook' );
            // we need to take it again since it could have been updated by the previous update
            
            if ( !empty($cook_opts) ) {
                // here we add "optin" and "optout" options based on the options in the DB
                // we do not change "enable_scripts_after" option
                // it doesn't matter whether the geo is enabled or not - admin might have enabled geo without changing & saving cookie notice options
                // GET CURRENT OPTIONS
                // get where to show the notice
                $show_notice_in = 'all countries';
                
                if ( isset( $cook_opts['show_notice'] ) && $cook_opts['show_notice'] == 'in_countries' && !empty($cook_opts['show_to_countries']) ) {
                    $show_notice_in = 'some countries';
                    $show_countries_str = $cook_opts['show_to_countries'];
                    $show_countries_str_trimmed = str_replace( ' ', '', $show_countries_str );
                    $show_countries_arr = explode( ',', $show_countries_str_trimmed );
                }
                
                // get where to require consent
                $require_consent_in = 'all countries';
                
                if ( $cook_opts['enable_scripts_after'] == 'pageload' ) {
                    $require_consent_in = 'nowhere';
                } else {
                    
                    if ( $cook_opts['enable_scripts_after'] == 'auto' ) {
                        $require_consent_in = 'some countries';
                        $req_consent_str = 'AT, BE, BG, CY, CZ, DE, DK, ES, EE, FI, FR, GB, GR, HR, HU, IE, IT, LT, LU, LV, MT, NL, PL, PT, RO, SK, SI, SE, MX, NG';
                        // this string is used below
                        $req_consent_str_trimmed = str_replace( ' ', '', $req_consent_str );
                        $req_countries_arr = explode( ',', $req_consent_str_trimmed );
                    } else {
                        
                        if ( $cook_opts['enable_scripts_after'] == 'choice' && !empty($cook_opts['req_consent_in_countries']) ) {
                            $require_consent_in = 'some countries';
                            $req_consent_str = $cook_opts['req_consent_in_countries'];
                            $req_consent_str_trimmed = str_replace( ' ', '', $req_consent_str );
                            $req_countries_arr = explode( ',', $req_consent_str_trimmed );
                        }
                    
                    }
                
                }
                
                // "TRANSLATE" CURRENT OPTIONS INTO "OPTIN" AND "OPTOUT"
                $optin = 'all';
                $optin_countries = '';
                $optout = 'none';
                $optout_countries = '';
                
                if ( $show_notice_in == 'all countries' && $require_consent_in == 'nowhere' ) {
                    $optin = 'none';
                    $optout = 'rest';
                } else {
                    
                    if ( $show_notice_in == 'all countries' && $require_consent_in == 'some countries' ) {
                        $optin = 'specific';
                        $optin_countries = $req_consent_str;
                        $optout = 'rest';
                    } else {
                        
                        if ( $show_notice_in == 'some countries' && $require_consent_in == 'all countries' ) {
                            // this situation is only when the user makes an error when setting things up
                            $optin = 'specific';
                            $optin_countries = $show_countries_str;
                        } else {
                            
                            if ( $show_notice_in == 'some countries' && $require_consent_in == 'nowhere' ) {
                                // we load tools on pageload in some countries and don't show the notice in the rest
                                $optin = 'none';
                                $optout = 'specific';
                                $optout_countries = $show_countries_str;
                            } else {
                                
                                if ( $show_notice_in == 'some countries' && $require_consent_in == 'some countries' ) {
                                    // it is possible that the user made an error while specyfying countries
                                    $optin = 'specific';
                                    $optin_countries_arr = array_intersect( $show_countries_arr, $req_countries_arr );
                                    $optin_countries = implode( ', ', $optin_countries_arr );
                                    $optout = 'specific';
                                    $optout_countries_arr = array_diff( $show_countries_arr, $optin_countries_arr );
                                    $optout_countries = implode( ', ', $optout_countries_arr );
                                }
                            
                            }
                        
                        }
                    
                    }
                
                }
                
                $cook_opts['optin'] = $optin;
                // all, none or specific
                if ( !empty($optin_countries) ) {
                    $cook_opts['optin_countries'] = $optin_countries;
                }
                $cook_opts['optout'] = $optout;
                // rest, none or specific
                if ( !empty($optout_countries) ) {
                    $cook_opts['optout_countries'] = $optout_countries;
                }
                update_option( 'fupi_cook', $cook_opts );
            }
        
        }
    
    }
    
    private function load_dependencies()
    {
        // STANDARD FUPI
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-fupi-loader.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-fupi-i18n.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-fupi-notices.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-fupi-admin.php';
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-fupi-public.php';
        $this->loader = new Fupi_Loader();
    }
    
    private function set_locale()
    {
        $plugin_i18n = new Fupi_i18n();
        $this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'fupi_load_plugin_textdomain' );
    }
    
    private function define_admin_hooks()
    {
        $plugin_admin = new Fupi_Admin( $this->get_plugin_name(), $this->get_version() );
        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'fupi_enqueue_scripts' );
        $this->loader->add_action( 'admin_menu', $plugin_admin, 'fupi_add_admin_page' );
        $this->loader->add_action( 'admin_menu', $plugin_admin, 'fupi_add_plausible_stats_page' );
        $this->loader->add_action( 'admin_init', $plugin_admin, 'fupi_register_settings' );
        $this->loader->add_action( 'admin_init', $plugin_admin, 'fupi_activation_redirect' );
        $this->loader->add_action( 'admin_init', $plugin_admin, 'fupi_admin_notices' );
        $this->loader->add_action( 'customize_register', $plugin_admin, 'fupi_customize_register' );
        $this->loader->add_action( 'customize_preview_init', $plugin_admin, 'fupi_customizer_preview_scripts' );
    }
    
    private function define_public_hooks()
    {
        $plugin_public = new Fupi_Public( $this->get_plugin_name(), $this->get_version() );
        // add "page labels" taxonomy
        if ( !empty($this->tools) && isset( $this->tools['labelpages'] ) ) {
            $this->loader->add_action( 'init', $plugin_public, 'fupi_page_labels' );
        }
        $this->loader->add_action( 'init', $plugin_public, 'fupi_add_shortcodes' );
        $this->loader->add_action(
            'wp_head',
            $plugin_public,
            'fupi_get_data',
            -1
        );
        // includes all the "Get" scripts
        $this->loader->add_action(
            'wp_head',
            $plugin_public,
            'fupi_output_in_head',
            50
        );
        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'fupi_enqueue_public_scripts' );
        $this->loader->add_action( 'wp_footer', $plugin_public, 'fupi_output_in_footer' );
        // load scripts asynchronously
        if ( !empty($this->main) && isset( $this->main['async_scripts'] ) ) {
            $this->loader->add_filter(
                'script_loader_tag',
                $plugin_public,
                'fupi_async_scripts',
                10,
                3
            );
        }
        // geolocation
        $this->loader->add_action( 'wp_ajax_nopriv_fupi_ajax_geo', $plugin_public, 'fupi_ajax_geo' );
        $this->loader->add_action( 'wp_ajax_fupi_ajax_geo', $plugin_public, 'fupi_ajax_geo' );
    }
    
    public function run()
    {
        $this->loader->run();
    }

}