<?php

class Fupi_Public {

	private $plugin_name;
	private $version;

	public function __construct( $plugin_name, $version ) {

		$this->plugin_name 					= $plugin_name;
		$this->version 						= $version;

		// tools & main opts
		$this->main 						= get_option('fupi_main');
		$this->tools 						= get_option('fupi_tools');
		if ( empty( $this->tools ) ) return;

		// cookie notice opts
		$this->cook 						= get_option('fupi_cook');

		// other opts
		$this->woo 							= get_option('fupi_woo');
		$this->trackmeta					= get_option('fupi_trackmeta');
		$this->track404						= get_option('fupi_track404');
		$this->geo							= get_option('fupi_geo');

		// Cookie notice
		$this->cookie_notice_enabled 		= isset( $this->tools['cookie_notice'] ) || isset( $this->tools['cook'] );

		// Integrations
		$this->ga41_enabled					= isset( $this->tools['ga41'] );
		$this->ga42_enabled					= isset( $this->tools['ga42'] );
		$this->ga1_enabled 					= isset( $this->tools['ga1'] );
		$this->ga2_enabled 					= isset( $this->tools['ga2'] );
		$this->goptimize_enabled            = isset( $this->tools['gopt'] );
		$this->gads_enabled 				= isset( $this->tools['gads'] );
		$this->mads_enabled 				= isset( $this->tools['mads'] );
		$this->fbp1_enabled 				= isset( $this->tools['fbp1'] );
		$this->fbp2_enabled 				= isset( $this->tools['fbp2'] );
		$this->gtm_enabled 					= isset( $this->tools['gtm'] );
		$this->twit_enabled 				= isset( $this->tools['twit'] );
		$this->pin_enabled 					= isset( $this->tools['pin'] );
		$this->clar_enabled 				= isset( $this->tools['clar'] );
		$this->linkd_enabled 				= isset( $this->tools['linkd'] );
		$this->tik_enabled	 				= isset( $this->tools['tik'] );
		$this->hotj_enabled 				= isset( $this->tools['hotj'] );
		$this->pla_enabled 					= isset( $this->tools['pla'] );
		$this->sbee_enabled 				= isset( $this->tools['sbee'] );
		$this->inspectlet_enabled 			= isset( $this->tools['insp'] );
		$this->crazyegg_enabled 			= isset( $this->tools['cegg'] );
		$this->cscr_enabled 				= isset( $this->tools['cscr'] );
		$this->geo_enabled					= isset( $this->tools['geo'] );

		// Settings
		$this->noscript_support				= isset( $this->main['noscript'] );
		$this->is_debug						= isset( $this->main['debug'] );

		// Other
		$this->gtag_id						= false;

		if ( $this->ga1_enabled ) {

			$ga1_opts 						= get_option('fupi_ga1');
			$this->gtag_id 					= isset( $ga1_opts['tracking_id'] ) ? esc_attr( $ga1_opts['tracking_id'] ): false;

		} else if ($this->ga41_enabled) {

			$ga41_opts 						= get_option('fupi_ga41');
			$this->gtag_id 					= isset( $ga41_opts['id'] ) ? esc_attr( $ga41_opts['id'] ) : false;

		}

	}

	public function fupi_get_data () {

		if ( empty( $this->tools ) ) return;

		// GET THE DATA

		global $wp;
		global $post;

		$this->is_customizer 		= is_customize_preview();
		$this->user_is_logged_in 	= is_user_logged_in();
		$fp = [ 'loaded' => [] ];
		$fpdata = [];
		$cookie_notice = [];

		include_once dirname(__FILE__) . '/modules/in_head/data-main.php';
		include_once dirname(__FILE__) . '/modules/in_head/data-wp.php';
		if ( ! empty( $this->woo_enabled ) ) include_once dirname(__FILE__) . '/modules/in_head/data-woo__premium_only.php';
		include_once dirname(__FILE__) . '/modules/in_head/data-tools.php';

		$fp = apply_filters('fupi_modify_fp_object', $fp );
		$fpdata = apply_filters('fupi_modify_fpdata_object', $fpdata );

		// OUTPUT THE DATA

		$output = '<!--noptimize--><script id=\'fp_data_js\' type="text/javascript" data-no-optimize="1">
			var fp_premium = ' . json_encode( fupi_fs()->can_use_premium_code() ) . ',
				FP = { \'fns\' : {} },
				fp = ' . json_encode( $fp ) . ',
				fpdata = ' . json_encode( $fpdata ) . ';';

			include_once dirname(__FILE__) . '/modules/in_head/head-js.php';
		$output .= '</script><!--/noptimize-->';

		echo $output;
	}

	public function fupi_enqueue_public_scripts() {

		if ( empty( $this->tools ) ) return;

		// JS HELPERS

		/* ^ */ wp_enqueue_script( 'fupi-helpers-js', plugin_dir_url( __FILE__ ) . 'helpers/fupi-helpers.js', array(), $this->version, false ); // also sets fpdata.cookies!
		/* _ */ wp_enqueue_script( 'fupi-helpers-footer-js', plugin_dir_url( __FILE__ ) . 'helpers/fupi-helpers-footer.js', array('fupi-helpers-js'), $this->version, true );

		// COOKIE NOTICE

		if ( $this->cookie_notice_enabled ) {

			// if ( $this->is_customizer || ( $this->track_current_user && ! $this->is_customizer ) ) {
				/* ^ */ wp_enqueue_style( 'fupi-cookie-notice', plugin_dir_url( __FILE__ ) . 'cookie-notice/css/fupi-cookie-notice.min.css', array(), $this->version, 'all' );
			// }

			if ( $this->is_customizer ) {
				/* _ */ wp_enqueue_script( 'fupi-customizer-cookie-notice-js', plugin_dir_url( __FILE__ ) . 'cookie-notice/js/fupi-customizer-cookie-notice.js', array( 'fupi-helpers-js', 'jquery' ), $this->version, true );
			} else /*if ( $this->track_current_user )*/ {
				/* _ */ wp_enqueue_script( 'fupi-cookie-notice-js', plugin_dir_url( __FILE__ ) . 'cookie-notice/js/fupi-cookie-notice.js', array( 'fupi-helpers-js', 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}

		// Don't load any tracking tools in the customizer
		if ( $this->is_customizer ) return;

		// GTM

		if ( $this->gtm_enabled ) {

			// $gtm_opts = get_option('fupi_gtm');

			// files below need to have typos! Litespeed cache will NOT combine files with "gtm" in their names
			/* ^ */ wp_enqueue_script( 'fupi-gtm-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-gotm.js', array( 'fupi-helpers-js' ), $this->version, false );
			/* _ */ wp_enqueue_script( 'fupi-gtm-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-gotm-footer.js', array( 'fupi-helpers-js', 'fupi-helpers-footer-js', 'fupi-gtm-head-js' ), $this->version, true );
		}

		// Google Optimize (loaded with optimize.js)

		if ( $this->goptimize_enabled ) {

			$this->gopts = get_option('fupi_gopts');

			if ( ! isset ( $this->gopts['main_load_method'] ) || $this->gopts['main_load_method'] != 'gtag' ) { // the "isset" part is for backwards compatibility with FP 1.2 that had no GTAG integration

				if ( $this->track_current_user || isset( $this->gopts['force_load'] ) ) {

					/* ^ */ wp_enqueue_script( 'fupi-gopt-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-goptimize.js', array( 'fupi-helpers-js' ), $this->version, false );
				}
			}
		}

		// GTAG / GOOGLE ANALYTICS / GOOGLE ADS

		if ( $this->ga1_enabled || $this->ga41_enabled || $this->gads_enabled ) {

			$reqs = array('fupi-helpers-js');

			// GOOGLE ANALYTICS (UA)

			if ( $this->ga1_enabled ) {

				$ga1_opts = get_option('fupi_ga1');

				if ( $this->track_current_user || isset( $ga1_opts['force_load'] ) ) {

					/* ^ */ wp_enqueue_script( 'fupi-ga-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-ga.js', array( 'fupi-helpers-js' ), $this->version, false );
					/* _ */ wp_enqueue_script( 'fupi-ga-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-ga-footer.js', array( 'fupi-ga-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
					array_push( $reqs, 'fupi-ga-head-js');
				}
			}

			// GOOGLE ANALYTICS 4

			if ( $this->ga41_enabled ) {

				$ga41_opts = get_option('fupi_ga41');

				if ( $this->track_current_user || isset( $ga41_opts['force_load'] ) ) {

					/* ^ */ wp_enqueue_script( 'fupi-ga4-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-ga4.js', array( 'fupi-helpers-js' ), $this->version, false );
					/* _ */ wp_enqueue_script( 'fupi-ga4-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-ga4-footer.js', array( 'fupi-ga4-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
					array_push( $reqs, 'fupi-ga4-head-js');
				}
			}

			// GOOGLE ADS

			if ( $this->gads_enabled ) {

				$gasd_opts = get_option('fupi_gads');

				if ( $this->track_current_user || isset( $gads_opts['force_load'] ) ) {

					/* ^ */ wp_enqueue_script( 'fupi-gads-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-gads.js', array( 'fupi-helpers-footer-js' ), $this->version, false );
					/* _ */ wp_enqueue_script( 'fupi-gads-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-gads-footer.js', array( 'fupi-gads-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
					array_push( $reqs, 'fupi-gads-head-js');
				}
			}

			// GTAG

			/* ^ */ wp_enqueue_script( 'fupi-gtg-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-gtg.js', $reqs, $this->version, false ); // this typo needs to be here! Litespeed cache will NOT combine files with "gtag" in its name
		}

		// FB PIXEL

		if ( $this->fbp1_enabled ) {

			$fbp1_opts = get_option('fupi_fbp1');

			if ( $this->track_current_user || isset( $fbp1_opts['force_load'] ) ) {

				/* ^ */ wp_enqueue_script( 'fupi-fbp-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-fbp.js', array( 'fupi-helpers-js' ), $this->version, false );
				/* _ */ wp_enqueue_script( 'fupi-fbp-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-fbp-footer.js', array( 'fupi-fbp-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}

		// TWITTER

		if ( $this->twit_enabled ) {

			$twit_opts = get_option('fupi_twit');

			if ( $this->track_current_user || isset( $twit_opts['force_load'] ) ) {

				/* ^ */ wp_enqueue_script( 'fupi-twit-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-twit.js', array( 'fupi-helpers-js' ), $this->version, false );
				/* _ */ wp_enqueue_script( 'fupi-twit-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-twit-footer.js', array( 'fupi-twit-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}

		// MS ADS

		if ( $this->mads_enabled ) {

			$mads_opts = get_option('fupi_mads');

			if ( $this->track_current_user || isset( $mads_opts['force_load'] ) ) {

				/* ^ */ wp_enqueue_script( 'fupi-mads-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-mads.js', array( 'fupi-helpers-js' ), $this->version, false );
				/* _ */ wp_enqueue_script( 'fupi-mads-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-mads-footer.js', array( 'fupi-mads-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}

		// PINTEREST

		if ( $this->pin_enabled ) {

			$pin_opts = get_option('fupi_pin');

			if ( $this->track_current_user || isset( $pin_opts['force_load'] ) ) {

				/* ^ */ wp_enqueue_script( 'fupi-pin-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-pin.js', array( 'fupi-helpers-js' ), $this->version, false );
			}
		}

		// LINKEDIN

		if ( $this->linkd_enabled ) {

			$linkd_opts = get_option('fupi_linkd');

			if ( $this->track_current_user || isset( $linkd_opts['force_load'] ) ) {

				/* _ */ wp_enqueue_script( 'fupi-linkd-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-linkd-footer.js', array( 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}

		// TIKTOK

		if ( $this->tik_enabled ) {

			$tik_opts = get_option('fupi_tik');

			if ( $this->track_current_user || isset( $tik_opts['force_load'] ) ) {

				/* ^ */ wp_enqueue_script( 'fupi-tik-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-tik.js', array( 'fupi-helpers-js' ), $this->version, false );
				/* _ */ wp_enqueue_script( 'fupi-tik-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-tik-footer.js', array( 'fupi-helpers-footer-js', 'fupi-tik-head-js' ), $this->version, true );
			}
		}

		// CLARITY

		if ( $this->clar_enabled ) {

			$clar_opts = get_option('fupi_clar');

			if ( $this->track_current_user || isset ( $clar_opts['force_load'] ) ) {

				/* ^ */ wp_enqueue_script( 'fupi-clar-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-clar.js', array( 'fupi-helpers-js' ), $this->version, false );
				/* _ */ wp_enqueue_script( 'fupi-clar-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-clar-footer.js', array( 'fupi-clar-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}

		// HOTJAR

		if ( $this->hotj_enabled ) {

			$hotj_opts = get_option('fupi_hotj');

			if ( $this->track_current_user || isset( $hotj_opts['force_load'] ) ) {
				/* ^ */ wp_enqueue_script( 'fupi-hotj-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-hotj.js', array( 'fupi-helpers-js' ), $this->version, false );
				/* _ */ wp_enqueue_script( 'fupi-hotj-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-hotj-footer.js', array( 'fupi-hotj-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}

		// INSPECTLET

		if ( $this->inspectlet_enabled ) {

			$insp_opts = get_option('fupi_insp');

			if ( $this->track_current_user || isset( $insp_opts['force_load'] ) ) {
				/* ^ */ wp_enqueue_script( 'fupi-insp-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-insp.js', array( 'fupi-helpers-js' ), $this->version, false );
				/* _ */ wp_enqueue_script( 'fupi-insp-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-insp-footer.js', array( 'fupi-insp-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}

		// CRAZYEGG

		if ( $this->crazyegg_enabled ) {

			$cegg_opts = get_option('fupi_cegg');

			if ( $this->track_current_user || isset( $cegg_opts['force_load'] ) ) {
				/* ^ */ wp_enqueue_script( 'fupi-cegg-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-cegg.js', array( 'fupi-helpers-js' ), $this->version, false );
				/* _ */ wp_enqueue_script( 'fupi-cegg-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-cegg-footer.js', array( 'fupi-cegg-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}

		// PLAUSIBLE

		if ( $this->pla_enabled ) {

			$pla_opts = get_option('fupi_pla');

			if ( $this->track_current_user || isset( $pla_opts['force_load'] ) ) {
				/* ^ */ wp_enqueue_script( 'fupi-pla-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-pla.js', array( 'fupi-helpers-js' ), $this->version, false );
				/* _ */ wp_enqueue_script( 'fupi-pla-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-pla-footer.js', array( 'fupi-pla-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}

		// SPLITBEE

		if ( $this->sbee_enabled ) {

			$sbee_opts = get_option('fupi_sbee');

			if ( $this->track_current_user || isset( $sbee_opts['force_load'] ) ) {
				/* ^ */ wp_enqueue_script( 'fupi-sbee-head-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-sbee.js', array( 'fupi-helpers-js' ), $this->version, false );
				/* _ */ wp_enqueue_script( 'fupi-sbee-footer-js', plugin_dir_url( __FILE__ ) . 'modules/js/fupi-sbee-footer.js', array( 'fupi-sbee-head-js', 'fupi-helpers-footer-js' ), $this->version, true );
			}
		}
	}

	function fupi_async_scripts( $tag, $handle, $src ) {

		// The handles of the enqueued scripts we want to defer
		$defer_scripts = array(
			'fupi-gtg-head-js',
			'fupi-sbee-head-js',
			'fupi-sbee-footer-js',
			'fupi-pla-head-js',
			'fupi-pla-footer-js',
			'fupi-cegg-head-js',
			'fupi-cegg-footer-js',
			'fupi-insp-head-js',
			'fupi-insp-footer-js',
			'fupi-hotj-head-js',
			'fupi-hotj-footer-js',
			'fupi-clar-head-js',
			'fupi-clar-footer-js',
			'fupi-tik-head-js',
			'fupi-tik-footer-js',
			'fupi-linkd-footer-js',
			'fupi-pin-head-js',
			'fupi-twit-head-js',
			'fupi-twit-footer-js',
			'fupi-mads-head-js',
			'fupi-mads-footer-js',
			'fupi-fbp-head-js',
			'fupi-fbp-footer-js',
			'fupi-ga4-head-js',
			'fupi-ga4-footer-js',
			'fupi-ga-head-js',
			'fupi-ga-footer-js',
			'fupi-gads-head-js',
			'fupi-gads-footer-js',
			'fupi-gopt-head-js',
			'fupi-gtm-head-js',
			'fupi-gtm-footer-js',
			'fupi-helpers-footer-js',
			'fupi-customizer-cookie-notice-js',
			// 'fupi-helpers-js', << can't be deferred since inline scripts use it
			// 'fupi-cookie-notice-js', << makes problems when deferred
			// 'fupi-woo-js', << makes problems when deferred
		);

	    if ( in_array( $handle, $defer_scripts ) ) {
	        return '<script src="' . $src . '" defer="defer" type="text/javascript"></script>' . "\n";
	    }

	    return $tag;
	}

	//
	// OUTPUT IN HEAD with PHP
	//

	public function fupi_output_in_head () {

		if ( empty( $this->tools ) ) return;

		// CUSTOM HEAD SCRIPTS
		if ( $this->cscr_enabled && ! $this->is_customizer ) {
			include_once dirname(__FILE__) . '/modules/in_head/custom-head-scripts.php';
		}
	}

	//
	// OUTPUT IN FOOTER with PHP
	//

	public function fupi_output_in_footer () {

		if ( empty( $this->tools ) ) return;

		// COOKIE NOTICE HTML
		if ( $this->cookie_notice_enabled ) {
			include_once dirname(__FILE__) . '/cookie-notice/fupi-display-cookie-notice.php';
		}

		// CUSTOM FOOTER SCRIPTS
		if ( $this->cscr_enabled &&! $this->is_customizer ) {
			include_once dirname(__FILE__) . '/modules/in_footer/custom-footer-scripts.php';
		}

		// NOSCRIPT TAGS
		if ( $this->noscript_support && $this->track_current_user && ! $this->is_customizer ) {
			include_once dirname(__FILE__) . '/modules/in_footer/noscript-fallbacks.php';
		}
	}

	//
	// SHORTCODES
	//

	public function fupi_add_shortcodes() {

		if ( empty( $this->tools ) ) return;

		add_shortcode( 'fp_info', array($this, 'fupi_info') );
		add_shortcode( 'fp_block', array($this, 'fupi_block') );
	}

	public function fupi_block($atts, $content = null){

		if ( $this->cookie_notice_enabled && isset( $this->tools['iframeblock'] ) ) {

			$a = shortcode_atts( array(
				'stats' => '',
				'market' => '',
				'pers' => '',
				'image' => false,
				'lazy' => false,
				'width' => '16',
				'height' => '9'
			), $atts );

			if ( empty( $content ) ){
				return '';
			} else {

				// get all the pieces
				$iframeblock = get_option('fupi_iframeblock');

				$stats = ! empty( $a['stats'] ) && $a['stats'] == '1' ? '1' : '0';
				$market = ! empty( $a['market'] ) && $a['market'] == '1' ? '1' : '0';
				$pers = ! empty( $a['pers'] ) && $a['pers'] == '1' ? '1' : '0';

				$img_src = '';

				if ( $a['image'] !== false ){
					if ( $a['image'] != 'none' ){
						$img_src = esc_url( $a['image'] );
					}
				} else if ( ! empty( $iframeblock['iframe_img'] ) ){
					$img_src = esc_url( $iframeblock['iframe_img'] );
				}

				$lazy = '';

				if ( $a['lazy'] == '1' ) {
					$lazy = ' loading="lazy"';
				} else if ( ! empty( $iframeblock['iframe_lazy'] ) ) {
					$lazy = ' loading="lazy"';
				}

				// let's put it all together

				// if we are dealing with an embed in gutenberg
				if ( str_contains( $content, 'wp-has-aspect-ratio' ) ){
					// we already have the aspect ratio so we only need to add a placeholder image if it is available
					$style = empty( $img_src ) ? '' : 'style="--fupi-iframe-ph: url(' . $img_src . ');"';
					$output = '<div class="fupi_block fupi_embed_wrap fupi_stats_' . $stats . ' fupi_market_' . $market . ' fupi_pers_' . $pers . '" ' . $style . '>';

				// if we have an HTML iframe
				} else {

					$style = '';

					// we display an image placeholder only if we have it
					if ( ! empty( $img_src ) ) {

						$style = 'style="--fupi-iframe-ph: url(' . $img_src . ');';

						// we make the aspect ratio only if we have an image
						if ( is_numeric( $a['height'] ) && is_numeric( $a['width'] ) ){
							$style .= ' --fupi-iframe-aspect: ' . (int) $a['height'] / (int) $a['width'] * 100 . '%;';
						}

						$style .= '"';
					} else {
						$style .= 'style="--fupi-iframe-aspect: ' . (int) $a['height'] / (int) $a['width'] * 100 . '%;"';
					}

					$output = '<div class="fupi_block fupi_simple_iframe fupi_resp_iframe fupi_stats_' . $stats . ' fupi_market_' . $market . ' fupi_pers_' . $pers . '" '. $style .'>';
				}

				$new_content = str_replace( '<iframe', '<div class="fupi_blocked_iframe"' . $lazy, $content );
				$output .= str_replace( '/iframe>', '/div>', $new_content ) . '</div><script>FP.unblock();</script>';

				return $output;
			}
		} else {
			return $content; // this returns only iframes - shortcodes are always invisible ( is saves user time removing them if the iframe blocking module was disabled )
		}
	}

	public function fupi_info ( $atts, $content = null ) {

		if ( ! empty( $this->tools['privex'] ) ) {
			$a = shortcode_atts( array(
				'display' => 'list',
			), $atts );

			$output = '<ol class="fupi_privacy fupi_display_as_' . esc_attr($a['display']) . '">';

				if ( $this->ga41_enabled || $this->ga1_enabled ) {
					$output .= '<li>Google Analytics</li>';
				}
				if ( $this->fbp1_enabled ) {
					$output .= '<li>Facebook</li>';
				}
				if ( $this->goptimize_enabled ) {
					$output .= '<li>Google Optimize</li>';
				}
				if ( $this->twit_enabled ) {
					$output .= '<li>Twitter</li>';
				}
				if ( $this->gads_enabled ) {
					$output .= '<li>Google Ads</li>';
				}
				if ( $this->mads_enabled ) {
					$output .= '<li>Microsoft Advertising</li>';
				}
				if ( $this->pin_enabled ) {
					$output .= '<li>Pinterest</li>';
				}
				if ( $this->linkd_enabled ) {
					$output .= '<li>Linked In</li>';
				}
				if ( $this->clar_enabled ) {
					$output .= '<li>Microsoft Clarity</li>';
				}
				if ( $this->hotj_enabled ) {
					$output .= '<li>Hotjar</li>';
				}
				if ( $this->crazyegg_enabled ) {
					$output .= '<li>Crazy Egg</li>';
				}
				if ( $this->sbee_enabled ) {
					$sbee_opts = get_option('fupi_sbee');
					if ( !empty( $sbee_opts ) && ! isset( $sbee_opts['no_cookies'] ) ){
						$output .= '<li>Splitbee</li>';
					}
				}
				if ( $this->inspectlet_enabled ) {
					$output .= '<li>Inspectlet</li>';
				}
				if ( $this->cscr_enabled ){
					$scripts_a = get_option('fupi_cscr');

					if ( empty( $scripts_a ) ) return;

					if ( is_array($scripts_a) ){

						if ( ! empty( $scripts_a['fupi_head_scripts'] ) && is_array( $scripts_a['fupi_head_scripts'] ) ){
							foreach( $scripts_a['fupi_head_scripts'] as $script_data ){
								if ( ! empty( $script_data['name'] ) ) $output .= '<li>'.esc_attr($script_data['name']).'</li>';
							}
						}

						if ( ! empty( $scripts_a['fupi_footer_scripts'] ) && is_array( $scripts_a['fupi_footer_scripts'] ) ){
							foreach( $scripts_a['fupi_footer_scripts'] as $script_data ){
								if ( ! empty( $script_data['name'] ) ) $output .= '<li>'.esc_attr($script_data['name']).'</li>';
							}
						}
					}
				}
				if ( isset( $this->tools['privex'] ) ){

					$privex_opts = get_option('fupi_privex');

					if ( ! empty( $privex_opts ) && ! empty( $privex_opts['extra_tools'] ) && is_array( $privex_opts['extra_tools'] ) ){
						foreach( $privex_opts['extra_tools'] as $tool ){
							if ( ! empty( $tool['url'] ) ){
								$output .= '<li><a href="'.esc_url($tool['url']).'" target="_blank" rel="nofollow">'.esc_attr($tool['name']).'</a></li>';
							} else {
								$output .= '<li>'.esc_attr($tool['name']).'</li>';
							}
						}
					}
				}

			$output .= '</ol>';

			return $output;
		} else {
			return '';
		}
	}

	//
	// GEO
	//

	public function fupi_ajax_geo() {

		$method = esc_attr( $_POST['method'] );

		if ( $method == 'cloudways' ) {

			$countries = array('Afghanistan'=>'AF', 'Aland Islands'=>'AX', 'Albania'=>'AL', 'Algeria'=>'DZ', 'American Samoa'=>'AS', 'Andorra'=>'AD', 'Angola'=>'AO', 'Anguilla'=>'AI', 'Antarctica'=>'AQ', 'Antigua And Barbuda'=>'AG', 'Argentina'=>'AR', 'Armenia'=>'AM', 'Aruba'=>'AW', 'Australia'=>'AU', 'Austria'=>'AT', 'Azerbaijan'=>'AZ', 'Bahamas'=>'BS', 'Bahrain'=>'BH', 'Bangladesh'=>'BD', 'Barbados'=>'BB', 'Belarus'=>'BY', 'Belgium'=>'BE', 'Belize'=>'BZ', 'Benin'=>'BJ', 'Bermuda'=>'BM', 'Bhutan'=>'BT', 'Bolivia'=>'BO', 'Bosnia And Herzegovina'=>'BA', 'Botswana'=>'BW', 'Bouvet Island'=>'BV', 'Brazil'=>'BR', 'British Indian Ocean Territory'=>'IO', 'Brunei Darussalam'=>'BN', 'Bulgaria'=>'BG', 'Burkina Faso'=>'BF', 'Burundi'=>'BI', 'Cambodia'=>'KH', 'Cameroon'=>'CM', 'Canada'=>'CA', 'Cape Verde'=>'CV', 'Cayman Islands'=>'KY', 'Central African Republic'=>'CF', 'Chad'=>'TD', 'Chile'=>'CL', 'China'=>'CN', 'Christmas Island'=>'CX', 'Cocos (Keeling) Islands'=>'CC', 'Colombia'=>'CO', 'Comoros'=>'KM', 'Congo'=>'CG', 'Congo, Democratic Republic'=>'CD', 'Cook Islands'=>'CK', 'Costa Rica'=>'CR', 'Cote D\'Ivoire'=>'CI', 'Croatia'=>'HR', 'Cuba'=>'CU', 'Cyprus'=>'CY', 'Czech Republic'=>'CZ', 'Denmark'=>'DK', 'Djibouti'=>'DJ', 'Dominica'=>'DM', 'Dominican Republic'=>'DO', 'Ecuador'=>'EC', 'Egypt'=>'EG', 'El Salvador'=>'SV', 'Equatorial Guinea'=>'GQ', 'Eritrea'=>'ER', 'Estonia'=>'EE', 'Ethiopia'=>'ET', 'Falkland Islands (Malvinas)'=>'FK', 'Faroe Islands'=>'FO', 'Fiji'=>'FJ', 'Finland'=>'FI', 'France'=>'FR', 'French Guiana'=>'GF', 'French Polynesia'=>'PF', 'French Southern Territories'=>'TF', 'Gabon'=>'GA', 'Gambia'=>'GM', 'Georgia'=>'GE', 'Germany'=>'DE', 'Ghana'=>'GH', 'Gibraltar'=>'GI', 'Greece'=>'GR', 'Greenland'=>'GL', 'Grenada'=>'GD', 'Guadeloupe'=>'GP', 'Guam'=>'GU', 'Guatemala'=>'GT', 'Guernsey'=>'GG', 'Guinea'=>'GN', 'Guinea-Bissau'=>'GW', 'Guyana'=>'GY', 'Haiti'=>'HT', 'Heard Island & Mcdonald Islands'=>'HM', 'Holy See (Vatican City State)'=>'VA', 'Honduras'=>'HN', 'Hong Kong'=>'HK', 'Hungary'=>'HU', 'Iceland'=>'IS', 'India'=>'IN', 'Indonesia'=>'ID', 'Iran, Islamic Republic Of'=>'IR', 'Iraq'=>'IQ', 'Ireland'=>'IE', 'Isle Of Man'=>'IM', 'Israel'=>'IL', 'Italy'=>'IT', 'Jamaica'=>'JM', 'Japan'=>'JP', 'Jersey'=>'JE', 'Jordan'=>'JO', 'Kazakhstan'=>'KZ', 'Kenya'=>'KE', 'Kiribati'=>'KI', 'Korea'=>'KR', 'Kuwait'=>'KW', 'Kyrgyzstan'=>'KG', 'Lao People\'s Democratic Republic'=>'LA', 'Latvia'=>'LV', 'Lebanon'=>'LB', 'Lesotho'=>'LS', 'Liberia'=>'LR', 'Libyan Arab Jamahiriya'=>'LY', 'Liechtenstein'=>'LI', 'Lithuania'=>'LT', 'Luxembourg'=>'LU', 'Macao'=>'MO', 'Macedonia'=>'MK', 'Madagascar'=>'MG', 'Malawi'=>'MW', 'Malaysia'=>'MY', 'Maldives'=>'MV', 'Mali'=>'ML', 'Malta'=>'MT', 'Marshall Islands'=>'MH', 'Martinique'=>'MQ', 'Mauritania'=>'MR', 'Mauritius'=>'MU', 'Mayotte'=>'YT', 'Mexico'=>'MX', 'Micronesia, Federated States Of'=>'FM', 'Moldova'=>'MD', 'Monaco'=>'MC', 'Mongolia'=>'MN', 'Montenegro'=>'ME', 'Montserrat'=>'MS', 'Morocco'=>'MA', 'Mozambique'=>'MZ', 'Myanmar'=>'MM', 'Namibia'=>'NA', 'Nauru'=>'NR', 'Nepal'=>'NP', 'Netherlands'=>'NL', 'Netherlands Antilles'=>'AN', 'New Caledonia'=>'NC', 'New Zealand'=>'NZ', 'Nicaragua'=>'NI', 'Niger'=>'NE', 'Nigeria'=>'NG', 'Niue'=>'NU', 'Norfolk Island'=>'NF', 'Northern Mariana Islands'=>'MP', 'Norway'=>'NO', 'Oman'=>'OM', 'Pakistan'=>'PK', 'Palau'=>'PW', 'Palestinian Territory,\' Occupied'=>'PS', 'Panama'=>'PA', 'Papua New Guinea'=>'PG', 'Paraguay'=>'PY', 'Peru'=>'PE', 'Philippines'=>'PH', 'Pitcairn'=>'PN', 'Poland'=>'PL', 'Portugal'=>'PT', 'Puerto Rico'=>'PR', 'Qatar'=>'QA', 'Reunion'=>'RE', 'Romania'=>'RO', 'Russian Federation'=>'RU', 'Rwanda'=>'RW', 'Saint Barthelemy'=>'BL', 'Saint Helena'=>'SH', 'Saint Kitts And Nevis'=>'KN', 'Saint Lucia'=>'LC', 'Saint Martin'=>'MF', 'Saint Pierre And Miquelon'=>'PM', 'Saint Vincent And Grenadines'=>'VC', 'Samoa'=>'WS', 'San Marino'=>'SM', 'Sao Tome And Principe'=>'ST', 'Saudi Arabia'=>'SA', 'Senegal'=>'SN', 'Serbia'=>'RS', 'Seychelles'=>'SC', 'Sierra Leone'=>'SL', 'Singapore'=>'SG', 'Slovakia'=>'SK', 'Slovenia'=>'SI', 'Solomon Islands'=>'SB', 'Somalia'=>'SO', 'South Africa'=>'ZA', 'South Georgia And Sandwich Isl.'=>'GS', 'Spain'=>'ES', 'Sri Lanka'=>'LK', 'Sudan'=>'SD', 'Suriname'=>'SR', 'Svalbard And Jan Mayen'=>'SJ', 'Swaziland'=>'SZ', 'Sweden'=>'SE', 'Switzerland'=>'CH', 'Syrian Arab Republic'=>'SY', 'Taiwan'=>'TW', 'Tajikistan'=>'TJ', 'Tanzania'=>'TZ', 'Thailand'=>'TH', 'Timor-Leste'=>'TL', 'Togo'=>'TG', 'Tokelau'=>'TK', 'Tonga'=>'TO', 'Trinidad And Tobago'=>'TT', 'Tunisia'=>'TN', 'Turkey'=>'TR', 'Turkmenistan'=>'TM', 'Turks And Caicos Islands'=>'TC', 'Tuvalu'=>'TV', 'Uganda'=>'UG', 'Ukraine'=>'UA', 'United Arab Emirates'=>'AE', 'United Kingdom'=>'GB', 'United States'=>'US', 'United States Outlying Islands'=>'UM', 'Uruguay'=>'UY', 'Uzbekistan'=>'UZ', 'Vanuatu'=>'VU', 'Venezuela'=>'VE', 'Viet Nam'=>'VN', 'Virgin Islands, British'=>'VG', 'Virgin Islands, U.S.'=>'VI', 'Wallis And Futuna'=>'WF', 'Western Sahara'=>'EH', 'Yemen'=>'YE', 'Zambia'=>'ZM', 'Zimbabwe'=>'ZW');

			$country = isset($_SERVER['HTTP_X_FORWARDED_COUNTRY']) ? $_SERVER['HTTP_X_FORWARDED_COUNTRY'] : false;

			if ( !empty( $countries[$country] ) ) {
				echo json_encode( array('success' => true, 'message' => $countries[$country] ) );
				die();
			} else {
				echo json_encode( array('success' => false, 'message' => 'Country couldn\'t be located' ) );
				die();
			};

		};

		echo json_encode( array('success' => false, 'message' => 'Unknown or unspecified geolocation method' ) );
		die();
	}

	//
	// CUSTOM TAXONOMY
	//

	public function fupi_page_labels() {

		$labels = array(
			'name'                       => _x( 'Labels', 'Taxonomy General Name', 'fupi' ),
			'singular_name'              => _x( 'Label', 'Taxonomy Singular Name', 'fupi' ),
			'menu_name'                  => __( 'Labels', 'fupi' ),
			'all_items'                  => __( 'All labels', 'fupi' ),
			'parent_item'                => __( 'Parent label', 'fupi' ),
			'parent_item_colon'          => __( 'Parent Item:', 'fupi' ),
			'new_item_name'              => __( 'New Label Name', 'fupi' ),
			'add_new_item'               => __( 'Add New Label', 'fupi' ),
			'edit_item'                  => __( 'Edit Label', 'fupi' ),
			'update_item'                => __( 'Update Label', 'fupi' ),
			'view_item'                  => __( 'View Label', 'fupi' ),
			'separate_items_with_commas' => __( 'Separate labels with commas', 'fupi' ),
			'add_or_remove_items'        => __( 'Add or remove labels', 'fupi' ),
			'choose_from_most_used'      => __( 'Choose from the most used', 'fupi' ),
			'popular_items'              => __( 'Popular labels', 'fupi' ),
			'search_items'               => __( 'Search labels', 'fupi' ),
			'not_found'                  => __( 'Not Found', 'fupi' ),
			'no_terms'                   => __( 'No labels', 'fupi' ),
			'items_list'                 => __( 'Labels list', 'fupi' ),
			'items_list_navigation'      => __( 'Labels list navigation', 'fupi' ),
		);
		$args = array(
			'labels'                     => $labels,
			'hierarchical'               => false,
			'public'                     => true,
			'show_ui'                    => true,
			'show_admin_column'          => true,
			'show_in_nav_menus'          => false,
			'show_tagcloud'              => false,
			'show_in_rest'               => true,
		);
		register_taxonomy( 'fupi_page_labels', array( 'page' ), $args );
	}

}
