"use strict";

(function() {

	'use strict';

	var notice = FP.findID('fupi_cookie_notice'),
		panel = {},
		setup_done = false;
		panel.welcome = FP.findID('fupi_welcome_panel');
		panel.settings = FP.findID('fupi_settings_panel');

	function setup_notice(){
		if ( fp.notice.use_custom_notice ){
			add_events_to_custom_notice_btns();
			show_notice_on_link_click(true);
		} else {
			add_events_to_default_notice_btns();
			if ( ! fpdata.cookies ){
				if ( fp.notice.show_here == 'yes' ) show_notice();
				show_notice_on_link_click();
			} else {
				show_notice_on_link_click(true);
			}
		}
		setup_done = true;
	}

	// SETUP COOKIE NOTICE

	setup_notice();

	document.addEventListener( 'fp_load_scripts', e=>{
		if ( ! setup_done ) setup_notice();
		// hide the notice if the user agreed or declined cookies in a different tab
		if ( fpdata.cookies && notice.classList.contains('fupi_fadeInUp') ) {
			hide_notice_wrapper();
			remove_blur();
			remove_scroll_lock();
		}
	} )

	// FUNCTIONS

	function remove_blur() {
		setTimeout( function(){
			var html_el = document.getElementsByTagName( 'html' )[0];
			if( html_el.classList.contains('fupi_blur') ) {
				html_el.classList.remove('fupi_blur');
				html_el.classList.add('fupi_blur_out');
			}
		}, 300 );
	}

	function lock_scroll() {
		if ( fp.notice.scroll_lock ) {
			var html_el = document.getElementsByTagName( 'html' )[0];
			html_el.classList.add('fupi_scroll_lock');
			document.body.style.overflowY = 'hidden';
		}
	}

	function remove_scroll_lock(){
		if ( fp.notice.scroll_lock ) {
			var html_el = document.getElementsByTagName( 'html' )[0];
			html_el.classList.remove('fupi_scroll_lock');
			document.body.style.overflowY = 'auto';
		}
	}

	function show_notice_on_link_click( set_switches = false ){

		let custom_settings_class = fp.notice.use_custom_notice && fp.notice.custom_cookies_settings_class ? ', ' + fp.notice.custom_cookies_settings_class : '',
			show_notice_links = FP.findAll('.fupi_show_cookie_notice, .fp_show_cookie_notice' + custom_settings_class);

		if(show_notice_links.length) {
			show_notice_links.forEach(function(link) {
				link.addEventListener('click', function(e) {
					e.preventDefault();
					if(set_switches) set_switches_state();
					show_notice_wrapper();
					panel.settings ? show_panel('settings') : show_panel('welcome');
				});
			});
		}
	}

	function set_switches_state() {
		var switches = FP.findAll(".fupi_switch input");

		if(switches.length > 0) {
			switches.forEach(function(the_switch) {
				if( fpdata.cookies && fpdata.cookies[the_switch.value]) the_switch.checked = true;
			});
		}
	}

	function add_events_to_custom_notice_btns(){

		let custom_accept_btn = fp.notice.use_custom_notice && fp.notice.custom_agree_cookies_class ? FP.findFirst( fp.notice.custom_agree_cookies_class ) : false,
			custom_decline_btn = fp.notice.use_custom_notice && fp.notice.custom_decline_cookies_class ? FP.findFirst( fp.notice.custom_decline_cookies_class ) : false;

		if( custom_accept_btn )
			custom_accept_btn.addEventListener( 'click', accept_all_cookies_and_fire_custom_event );

		if( custom_decline_btn )
			custom_decline_btn.addEventListener( 'click', decline_all_cookies );

	}

	function add_events_to_default_notice_btns() {

		let accept_all_btn = FP.findID('fupi_agree_to_all_cookies_btn'),
			decline_btn = FP.findID('fupi_decline_cookies_btn'),
			cookie_settings_btn = FP.findID('fupi_cookie_settings_btn'),
			accept_chosen_btn = FP.findID('fupi_agree_to_selected_cookies_btn'),
			return_btn = FP.findID('fupi_return_btn'),
			switches = FP.findAll('.fupi_switch');

		if( accept_all_btn ) {
			accept_all_btn.addEventListener('click', function(){

				// hide all
				hide_panel('welcome');
				hide_notice_wrapper();
				remove_blur();
				remove_scroll_lock();

				// save cookies and fire events
				accept_all_cookies_and_fire_custom_event(); // FP.unblock() is there
			});
		}

		if( decline_btn ) {
			decline_btn.addEventListener('click', function(){

				// hide all
				hide_panel('welcome');
				hide_notice_wrapper();
				remove_blur();
				remove_scroll_lock();

				// save cookies and fire events
				decline_all_cookies();
			});
		}

		if( accept_chosen_btn ) {
			accept_chosen_btn.addEventListener('click', function(){

				// hide all
				hide_panel('settings');
				hide_notice_wrapper();
				remove_blur();
				remove_scroll_lock();

				// save cookies and fire events
				if ( fp.vars.track_current_user ) accept_chosen_cookies();
				FP.unblock();
			});
		}

		if( cookie_settings_btn ) {
			cookie_settings_btn.addEventListener('click', function() {
				hide_panel('welcome');
				show_panel('settings');
			});
		}

		if( return_btn ) {
			return_btn.addEventListener('click', function() {
				hide_panel('settings');
				show_panel('welcome');
			});
		}

		if( switches ) {
			switches.forEach(function(the_switch) {
				the_switch.addEventListener('click', function(e) {
					toggle_switch(e.target);
				}, false);
			});
		}
	}

	// ACCESSIBLE SWITCH
	// https://developer.mozilla.org/en-US/docs/Web/Accessibility/ARIA/Roles/Switch_role

	function toggle_switch(target) {
		if(target.getAttribute("aria-checked") == "true") {
			target.setAttribute("aria-checked", "false");
		} else {
			target.setAttribute("aria-checked", "true");
		}
	}

	function accept_all_cookies_and_fire_custom_event() {

		if ( fp.vars.track_current_user ) {

			var newCookieVal = {
				'stats': true,
				'personalisation': true,
				'marketing': true,
				'disabled': false
			};

			FP.setCookie('fp_cookie', JSON.stringify(newCookieVal), 182);
			fpdata.cookies = newCookieVal;

			FP.sendEvt('fp_load_scripts', ['all']);
			FP.sendEvt('fupi_popup_closed', 'all accepted');
		}

		FP.unblock(); // unblock iframes
	}

	function accept_chosen_cookies() {

		let stats_checkbox = FP.findID('fupi_stats_agree'),
			perso_checkbox = FP.findID('fupi_pers_agree'),
			marke_checkbox = FP.findID('fupi_marketing_agree'),
			some_scripts_are_loaded = fp.loaded.length > 0,
			newCookieVal = {
				'stats': stats_checkbox ? stats_checkbox.checked : false,
				'personalisation': perso_checkbox ? perso_checkbox.checked : false,
				'marketing': marke_checkbox ? marke_checkbox.checked : false,
				'disabled': false
			};

		FP.setCookie('fp_cookie', JSON.stringify(newCookieVal), 182);

		fpdata.cookies = newCookieVal;

		if(some_scripts_are_loaded) {
			location.reload();
		} else {
			FP.sendEvt('fp_load_scripts', ['all']);
			FP.sendEvt('fupi_popup_closed', 'some accepted');
		}
	}

	function decline_all_cookies() {

		if ( fp.vars.track_current_user ) {

			let some_scripts_are_loaded = fp.loaded.length > 0,
				newCookieVal = {
					'stats': false,
					'personalisation': false,
					'marketing': false,
					'disabled': false
				};

			FP.setCookie('fp_cookie', JSON.stringify(newCookieVal), 182);
			fpdata.cookies = newCookieVal;
			FP.sendEvt('fupi_popup_closed', 'declined');

			if( some_scripts_are_loaded ) location.reload();
		}
	}

	function show_panel(type) {
		if(panel[type]) {
			panel[type].classList.remove('fupi_fadeOutDown');
			panel[type].classList.remove('fupi_hidden');
			panel[type].classList.add('fupi_animated');
			panel[type].classList.add('fupi_fadeInUp');
		}
	}

	function hide_panel(type) {
		panel[type].classList.remove('fupi_fadeInUp');
		panel[type].classList.add('fupi_animated');
		panel[type].classList.add('fupi_fadeOutDown');
	}

	function show_notice_wrapper() {
		notice.classList.remove('fupi_fadeOutDown');
		notice.classList.remove('fupi_hidden');
		notice.classList.add('fupi_fadeInUp');
		notice.classList.add('fupi_animated');
	}

	function hide_notice_wrapper() {
		notice.classList.remove('fupi_fadeInUp');
		notice.classList.add('fupi_fadeOutDown');
	}

	function show_notice() {
		lock_scroll();
		show_notice_wrapper();
		show_panel('welcome');
	}

})();
