(function (FP) {

    'use strict';

    var d = document,
        w = window;

    FP.v = {};
    fp.track_queue = [];
    fpdata.activity = { total : 0, last : 0, current : 0 };

    // extend 1 obj with another
    FP.extend = function (source, properties) {
        var property;
        for (property in properties) {
            if (properties.hasOwnProperty(property)) {
                source[property] = properties[property];
            }
        }
        return source;
    };

    // fires function every X ms with an option to fire them ADDITIONALLY on start ("leading") and after events ("trailing")
    FP.throttle = function (func, wait, options) {
        var context, args, result;
        var timeout = null;
        var previous = 0;
        if (!options) options = {};

        var later = function later() {
            previous = options.leading === false ? 0 : Date.now();
            timeout = null;
            result = func.apply(context, args);
            if (!timeout) context = args = null;
        };

        return function () {
            var now = Date.now();
            if (!previous && options.leading === false) previous = now;
            var remaining = wait - (now - previous);
            context = this;
            args = arguments;

            if (remaining <= 0 || remaining > wait) {
                if (timeout) {
                    clearTimeout(timeout);
                    timeout = null;
                }

                previous = now;
                result = func.apply(context, args);
                if (!timeout) context = args = null;
            } else if (!timeout && options.trailing !== false) {
                timeout = setTimeout(later, remaining);
            }

            return result;
        };
    };

    FP.isObjectEmpty = function (obj) {
        return obj.constructor === Object && Object.keys(obj).length === 0;
    };

    FP.isNumeric = function (n) {
        return !isNaN(parseFloat(n)) && isFinite(n);
    };

    // This helps prevent writing long condition checks like "if (fp.raw && fp.raw['woo all product impressions'] && fp.raw['woo all product impressions']['currency'] )"
    // use it like this FP.getInner(['fp','raw','woo all product impressions','currency'])
    FP.getInner = function (vals, splitter) {

        let args = Array.isArray(vals) ? vals : vals.split(splitter),
            obj = window[args.shift()];

        for (var i = 0; i < args.length; i++) {
            var prop = args[i].trim();
            if (!obj || !obj.hasOwnProperty(prop)) return false;
            obj = obj[prop];
        };

        return obj;
    };

    FP.reduce = function( arr ){
        return arr.reduce( function(accumulator, val){ return val ? accumulator + val : accumulator } );
    };

    FP.remove = function (el) {
        if (el) el.parentNode.removeChild(el);
    };

    FP.keepChecking = function (rule, every, until, cb, args) {
        args ? args : false;
        var timer = 0,
        stopChecking = setInterval(function () {
            if (rule()) {
                cb.call(this, args);
                clearInterval(stopChecking);
            }

            timer += every;

            if (timer == until) {
                clearInterval(stopChecking);
            }
        }, every);
    };

    FP.findFirst = function (e, c) {
        if ( !e ) return false;
        c = c || document;
        return c.querySelector(e);
    };

    FP.findID = function (e, c) {
        if ( !e ) return false;
        c = c || document;
        return c.getElementById(e);
    };

    FP.findClosestLinkEl = function (el) {
        if ( el.href ) {
            return el;
        } else {
            var closest_link = el.closest('a');
            return closest_link && closest_link.href ? closest_link : false; // make sure that <a> has an "href"
        }
    };

    function shouldCustomScriptLoadHere( geo ){

        let geo_method = geo[0],
            geo_countries = geo[1];

        if ( fp.geo && geo_method && geo_countries ) {
            if ( fpdata.country != 'unknown' && fpdata.country != 'checking' ){
                if ( geo_method == 'excl' ) {
                    if ( FP.isIn( fpdata.country, geo_countries ) ) return false;
                } else {
                    if ( ! FP.isIn( fpdata.country, geo_countries ) ) return false;
                }
            }
        }

        return true;
    }

    function shouldScriptLoadHere( id ){

        if ( fp.geo && fp[id] && fp[id].limit_country){
            if ( fpdata.country != 'unknown' && fpdata.country != 'checking' ) {
                if ( fp[id].limit_country.method == 'excl' ) {
                    if ( FP.isIn( fpdata.country, fp[id].limit_country.countries ) ) return false;
                } else {
                    if ( ! FP.isIn( fpdata.country, fp[id].limit_country.countries ) ) return false;
                }
            }
        };

        return true;
    };

    FP.isAllowedToLoad_basic = function( id, force_load, permissions_a, geo ){

        if ( fp.vars.debug ) console.log('Checking ' + id + ' loading conditions');

        // STOP if the script has already loaded
        if ( FP.isIn( id, fp.loaded ) ) return false;

        // START if the load is forced
        if ( force_load ) return true;

        // STOP if the tab is not in focus
        if ( document.hidden ) return false;

        // STOP tracking an excluded visitor (MUST be after checking FORCE LOAD)
        if ( ! fp.vars.track_current_user ) return false;

        // STOP if the script is set NOT to load in the current location
        if ( ! shouldCustomScriptLoadHere( geo ) ) return false;

        // STOP if any permissions need to be given in this location...
        if ( ! fpdata.cookies && fp.notice.wait_for_consent == 'yes' ) {
            if ( permissions_a.length > 0 ) return false;
        }

        // STOP if required permissions are not given
        if ( fpdata.cookies && permissions_a.length > 0 ) {
            if ( ! permissions_a.every( permission => fpdata.cookies[permission] ) ) return false;
        }

        return true;
    };

    FP.isAllowedToLoad = function( module_id, permissions_a, required_a, nr, tool_does_not_use_cookies = false ) {

        var id = nr ? module_id + nr : module_id;

        if ( fp.vars.debug ) console.log('Checking ' + id + ' loading conditions');

        // STOP if we have no data
        if ( ! fp[id] ) return false;

        // STOP if the script has already loaded
        if ( FP.isIn( id, fp.loaded ) ) return false;

        // STOP if the required data is missing
        if ( required_a.some( req => ! fp[id][req] ) ) return false;

        // START if the load is forced
        if ( fp[id].force_load ) return true;

        // STOP if the tab is not in focus
        if ( document.hidden ) return false;

        // STOP tracking an excluded visitor (MUST be after checking FORCE LOAD)
        if ( ! fp.vars.track_current_user ) return false;

        // STOP if the script is set NOT to load in the current location
        if ( ! shouldScriptLoadHere( id ) ) return false;

        // If we wait for consent in this location...
        if ( ! fpdata.cookies && fp.notice.wait_for_consent == 'yes' ) {

            // START if the tool doesn't use cookies
            if ( tool_does_not_use_cookies ) return true;

            // START if the tool is set to load on pageload disregarding cookies
            if ( fp[id].disreg_cookies ) return true;

            // STOP if any permissions need to be given
            if ( permissions_a.length > 0 ) return false;
        }

        // If user chose their cookie preferences
        if ( fpdata.cookies && permissions_a.length > 0 ) {
            // .. but some required permissions are missing
            if ( ! permissions_a.every( permission => fpdata.cookies[permission] ) ) {
                // START if the tool doesn't use cookies
                if ( tool_does_not_use_cookies ) return true;
                // STOP if it can't
                return false;
            };
        }

        return true;
    };

    FP.unblock = function(){

        let blocked_iframes = FP.findAll('.fupi_block'), // actually these are wrappers of the iframes or embeds
            iframes_to_load = [];

        if ( blocked_iframes.length > 0 ) {

            // if the visitor agreed to cookies
            if ( fpdata.cookies ) {
                // load only the ones that match requirements
                let iframe_cookie_1 = blocked_iframes.filter( frm => fpdata.cookies.stats ? true : frm.classList.contains('fupi_stats_0') );
                let iframe_cookie_2 = iframe_cookie_1.filter( frm => fpdata.cookies.personalisation ? true : frm.classList.contains('fupi_pers_0') );
                iframes_to_load = iframe_cookie_2.filter( frm => fpdata.cookies.marketing ? true : frm.classList.contains('fupi_market_0') )

            // if the visitor didn't agree to cookies
            } else {
                // load only the ones that don't require any
                iframes_to_load = blocked_iframes.filter( frm => frm.classList.contains('fupi_stats_0') && frm.classList.contains('fupi_pers_0') && frm.classList.contains('fupi_market_0') )
            }
        }

        iframes_to_load.forEach( frame_wrap => {

            let img = FP.findFirst('img', frame_wrap),
                blocked_iframe = FP.findFirst('.fupi_blocked_iframe', frame_wrap);

            if ( img ) img.remove();

            if ( blocked_iframe ) {

                let orig_html = blocked_iframe.outerHTML,
                    new_html = orig_html.replaceAll('<div class="fupi_blocked_iframe"','<iframe');

                new_html = new_html.replaceAll('/div>','/iframe>');

                blocked_iframe.insertAdjacentHTML('afterend', new_html);
                blocked_iframe.remove();

                frame_wrap.classList.remove('fupi_block');
                frame_wrap.classList.add('fupi_iframe_loaded');
            }

        })
    }

    FP.formatTextarea = function (module, address) {

        if ( fp[module][address] ){
            var temp_a = fp[module][address].split("\n").map(function (line) {
                var trimmed = line.trim();
                return trimmed.length > 0 ? trimmed.split('@') : false;
            });

            fp[module][address] = temp_a.filter( function ( el ) {
                if ( el !== false ) return el;
            });
        }
    };

    // the difference between this Fn and isAnchorLinkToCurrentPage is that this one doesn't check if the link actually works
    FP.isLinkToCurrentPage = function (link) {
        let target_url_obj = new URL( link.href );
        return target_url_obj.origin + target_url_obj.pathname == location.origin + location.pathname;
    }

    


    FP.doActions = function( trigger, caller_slug ){
        if ( fp[trigger] ) fp[trigger].forEach( fn => fn( caller_slug ) );
    }

    FP.on = function( trigger, fn ){

        // save all actions in arrays
        var arr = Array.isArray(trigger) ? trigger : [trigger]; // make sure that trigger is in an array

        arr.forEach(function (t, i) {
            if ( !fp[t] ) fp[t] = [];
            fp[t].push( fn );
        });

        // Trigger instant actions instantly if DOM has loaded
        if ( trigger == 'instant' && d.readyState !== 'loading' ) fn('all');

    }

    // FIRE TAGS WHEN THE DOCUMENT GETS AND LOSES FOCUS

    function doPageShowActions(){
        if ( ! fp.vars.page_show_tracked ){
            fp.vars.page_show_tracked = true;
            fp.vars.page_hide_tracked = false;

            fpdata.doc_in_focus = true;
            
            var cookies = FP.readCookie('fp_cookie');
            fpdata.cookies = cookies ? JSON.parse(cookies) : false;
            FP.sendEvt( 'fp_load_scripts', ['all'] );
            if ( fp.page_in_focus ) FP.doActions('page_in_focus');
        }
    }

    function doPageHideActions(){
        if ( ! fp.vars.page_hide_tracked ) {
            fp.vars.page_hide_tracked = true;
            fp.vars.page_show_tracked = false;

            fpdata.doc_in_focus = false;
            
            if ( fp.close ) FP.doActions('close');
            fpdata.activity.current = 0;
        }
    }


    // pagehide/show fires on some events but is cross-browser
    // !! it fires BEFORE visibilitychanges
    // !! when the pageshow event fires the document obj doesn't have updated cookies or localStarage yet, thus we don't use it for "show"
    window.addEventListener( "pagehide", e => { doPageHideActions() }, false );
    // window.addEventListener( "pageshow", e => { doPageShowActions('pageshow') }, false ); visibilitychange is universal

    // beforeunload doesn't work with prev/next page buttons and has problems on mobiles
    window.addEventListener( "beforeunload", e => { doPageHideActions() }, false );

    // visibilitychanges fires on all events but has problems on mobiles and in safari
    // !! fires AFTER pagehide/show
    // !! when the visibilitychange event fires for "show" the document obj already has updated cookies and localStarage
    document.addEventListener('visibilitychange', function () {
        document.hidden ? doPageHideActions() :  doPageShowActions('visibilitychange');
        //if ( fp.page_visibility ) FP.doActions( 'page_visibility' );
    }, false);

    // Show an icon indicating that the current user is no longer tracked

	if ( fpdata.cookies && fpdata.cookies.disabled ) {
		document.addEventListener('DOMContentLoaded', function() {
			document.body.insertAdjacentHTML('afterbegin', '<div id=\'fupi_disabled\' title="Tracking disabled"><svg style="width: 20px;" focusable="false" data-prefix="fas" data-icon="eye-slash" class="svg-inline--fa fa-eye-slash fa-w-20" role="img" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 640 512"><path fill="currentColor" d="M320 400c-75.85 0-137.25-58.71-142.9-133.11L72.2 185.82c-13.79 17.3-26.48 35.59-36.72 55.59a32.35 32.35 0 0 0 0 29.19C89.71 376.41 197.07 448 320 448c26.91 0 52.87-4 77.89-10.46L346 397.39a144.13 144.13 0 0 1-26 2.61zm313.82 58.1l-110.55-85.44a331.25 331.25 0 0 0 81.25-102.07 32.35 32.35 0 0 0 0-29.19C550.29 135.59 442.93 64 320 64a308.15 308.15 0 0 0-147.32 37.7L45.46 3.37A16 16 0 0 0 23 6.18L3.37 31.45A16 16 0 0 0 6.18 53.9l588.36 454.73a16 16 0 0 0 22.46-2.81l19.64-25.27a16 16 0 0 0-2.82-22.45zm-183.72-142l-39.3-30.38A94.75 94.75 0 0 0 416 256a94.76 94.76 0 0 0-121.31-92.21A47.65 47.65 0 0 1 304 192a46.64 46.64 0 0 1-1.54 10l-73.61-56.89A142.31 142.31 0 0 1 320 112a143.92 143.92 0 0 1 144 144c0 21.63-5.29 41.79-13.9 60.11z"></path></svg></div><style>#fupi_disabled{position: fixed; left: 0; bottom: 0; color: #fff; background-color: rgba(0,0,0,.2); padding: 10px 5px; z-index: 10000;cursor: pointer;}#fupi_disabled:hover{background-color: rgba(0,0,0,.8);}</style>');

			var disabled_ico = FP.findID('fupi_disabled');

			disabled_ico.addEventListener('click', function (e) {
                FP.deleteCookie('fp_cookie');
                fpdata.cookies = false;
                document.location = location.origin;
            })
		});
	}

    

    if ( ! document.hidden) {
        
        fpdata.doc_in_focus = true;
        fp.vars.page_show_tracked = true;
        fp.vars.page_hide_tracked = false;
    };

})(FP);
